package client.clientState.worldState;


import shared.models.Attack;
import shared.models.PlayableCharacter;
import shared.models.SimpleZone;
import shared.network.messages.ChatMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.QuestionMessage;
import shared.network.messages.actions.ActionMessage;
import shared.network.messages.actions.IsDeadMessage;
import shared.network.messages.actions.JoinZoneMessage;
import shared.network.messages.actions.LeaveZoneMessage;
import shared.network.messages.actions.MovementMessage;
import shared.network.messages.actions.UpdateHealthMessage;
import shared.network.messages.actions.UpdatePlacementMessage;
import client.clientState.GameState;
import client.clientState.worldState.playerMenu.PlayerMenu;
import client.clientState.worldState.playerMenu.PlayerMenuReturnType;
import client.graphics.components.QuestionPanel;
import client.graphics.components.TravelPanel;
import client.network.NetworkHandlerMain;
/**
 * 
 * WorldState is the state when the player is logged in to the GameWorld.
 */
public class WorldState extends GameState {
	
	
	private WorldModel worldModel;
	
	private MainWorldPanel worldPanel;
	private QuestionPanel questionPanel;
	private PopupState popupState = PopupState.POPUP_INACTIVE;
	private HUDMenuPanel menuPanel;
	private PlayerMenu characterMenu;
	private ChatPanel chat;
	private boolean zoneLoaded;
	
	private boolean hasInitialZone;
	private boolean hasCharacter;
	private boolean hasSentZoneRequest;
	private boolean shouldForceRedraw;
	public WorldState(){
		worldModel = new WorldModel();
		zoneLoaded = false;
		
		hasInitialZone = false;
		hasCharacter = false;
		hasSentZoneRequest = false;
		shouldForceRedraw = false;
	}
	
	
	@Override
	public void receivedMessage(Message msg) {
		if(msg.getMessageType() == MessageType.OBJECT_MESSAGE){
			ObjectMessage objMsg = (ObjectMessage) msg;
			if(objMsg.getObjectType() == ObjectType.SIMPLE_ZONE){
				SimpleZone zone = (SimpleZone)objMsg.getObject();
				worldModel.addZone(zone);
				if(hasInitialZone == false){
					worldPanel = new MainWorldPanel(worldModel, zone.getZoneId(), this);
					menuPanel = new HUDMenuPanel(worldModel);
					chat = new ChatPanel();
					hasInitialZone = true;
					shouldForceRedraw = true;
				}else{
					worldPanel.setCurrentZone(zone.getZoneId());
					System.out.println("Set zone as "+zone.getZoneId());
					shouldForceRedraw = true;
					//TODO: fix so that user can't do anything while zone is loaded
				}
			}
			
		}
		else if(msg.getMessageType() == MessageType.ACTION_MESSAGE){
			ActionMessage actMsg = (ActionMessage)msg;
			switch(actMsg.getActionType()){
			case MOVEMENT:
				MovementMessage movMsg = (MovementMessage)actMsg;
				worldModel.updateMovement(movMsg.getActionId(),movMsg.getCurrentLocation(),movMsg.getTargetLocation());
				break;
			case JOIN_ZONE:
				JoinZoneMessage joinMsg = (JoinZoneMessage)actMsg;
				worldModel.getZone(joinMsg.getZoneId()).addPlayerCharacter(joinMsg.getActionId(),joinMsg.getCharacter());
				shouldForceRedraw = true;
				break;
			case LEAVE_ZONE:
				LeaveZoneMessage leaveMsg = (LeaveZoneMessage)actMsg;
				worldModel.currentZone().removePlayerCharacter(leaveMsg.getActionId());
				shouldForceRedraw = true;
				break;
			case UPDATE_HEALTH:
				UpdateHealthMessage upMsg = (UpdateHealthMessage)actMsg;
				worldModel.updateHealth(upMsg.getActionId(),upMsg.getCurrentHealth(),upMsg.isHit());
				break;
			case IS_DEAD:
				IsDeadMessage deadMsg = (IsDeadMessage)actMsg;
				worldModel.isDead(deadMsg.getActionId(),deadMsg.getNameOfKiller());
				break;
			case UPDATE_POSITION:
				UpdatePlacementMessage plMsg = (UpdatePlacementMessage)actMsg;
				System.out.println(plMsg.getNewLocation());
				worldModel.getCharacter().setCurrentLocation(plMsg.getNewLocation());
				worldModel.getCharacter().setTargetLocation(plMsg.getNewLocation());
				shouldForceRedraw = true;
				break;
			}
		}
		else if(msg.getMessageType() == MessageType.QUESTION_MESSAGE){
			QuestionMessage q = (QuestionMessage)msg;
			for(String str:q.getAnswers()){
				System.out.println(str);
			}
			questionPanel = new QuestionPanel(q.getQuestionText(),
						q.getAnswers(), q.getTimeLimit(), q.getId(), q.getQuestionHeader());
			popupState = PopupState.QUESTION_POPUP_ACTIVE;
			shouldForceRedraw = true;
		}else if(msg.getMessageType() == MessageType.CHATMESSAGE){
			System.out.println("ChatMessage!");
			chat.receiveChatMessage((ChatMessage)msg);
		}
	}

	@Override
	public GameState update(long elapsedtime) {
		if(!hasInitialZone && hasCharacter && !hasSentZoneRequest){
			client.network.NetworkHandlerMain.getInstance().sendRequestZoneMessage(worldModel.getCharacter().getRespawnZone());
			hasSentZoneRequest = true;
		}
		if(hasInitialZone){
			chat.update();
			if(popupState == PopupState.CHARACTER_SCREEN_ACTIVE){
				
				PlayerMenuReturnType playerUpdateResult = characterMenu.update();
				if(playerUpdateResult == PlayerMenuReturnType.REDRAW){
					shouldForceRedraw = true;
				}
				else if(playerUpdateResult == PlayerMenuReturnType.MENU_CLOSING){
					popupState = PopupState.POPUP_INACTIVE;
					shouldForceRedraw = true;
				}
			}
			else if(popupState == PopupState.POPUP_INACTIVE){
				
				PopupState nextState = menuPanel.update(elapsedtime);
				
				if(nextState == PopupState.CHARACTER_SCREEN_ACTIVE){
					
					popupState = nextState;
					characterMenu = new PlayerMenu(worldModel);
					shouldForceRedraw = true;
				}
				
				if(worldPanel.update(elapsedtime)){
					shouldForceRedraw = true;
				}
				
			}
			
			//updates the questionPanel if it exists
			if(questionPanel != null){
				questionPanel.Update(elapsedtime);
				//when the player answers a question, the questionPanel closes 
				if (questionPanel.isQuestionAnswered()){
					NetworkHandlerMain.getInstance().sendAnswerMessage(questionPanel.getSelectedAnswer(),questionPanel.getSelectedAnswerText(), questionPanel.getTimeLeft());
					System.out.println("The player answered : " + questionPanel.getSelectedAnswer());
					System.out.println("The player had " + questionPanel.getTimeLeft() +" seconds left");
					destoryQuestionPanel();
				}
			}
		}
	
		if(shouldForceRedraw){
			shouldForceRedraw = false;
			forceDraw();
		}
		return this;
	}
	
	@Override
	public void forceDraw() {
		System.out.println("ForceRedraw");
		if(hasInitialZone){
			menuPanel.forceDraw();
			worldPanel.forceDraw();
			chat.forceDraw();
		}
		
		if(popupState == PopupState.CHARACTER_SCREEN_ACTIVE){
			characterMenu.forceDraw();
		}
		
		//draws the questionPanel
		if(questionPanel != null){
			client.graphics.ViewFacade.getInstance().addForegroundSprite(questionPanel);
		}
	
		
		client.graphics.ViewFacade.getInstance().updateBackground();
		client.graphics.ViewFacade.getInstance().updateMidBack();
		client.graphics.ViewFacade.getInstance().updateMidFront();
		client.graphics.ViewFacade.getInstance().updateForeground();
	}
	
	public void setCharacter(PlayableCharacter cha){
		worldModel.setCharacter(cha);
		hasCharacter = true;
	}
	
	/**
	 * creates a new questionPanel
	 * @param attack - the attack which will be used
	 * @param targetId  - the targes's id
	 */
	public void createQuestionPanel(Attack attack, String targetId){
		NetworkHandlerMain.getInstance().sendRequestQuestion(attack, targetId);
		System.out.println("Waiting for an question");
	}
	
	/**
	 * Destroys the questionPanel from the screen
	 */
	private void destoryQuestionPanel(){
		questionPanel = null;
		popupState = PopupState.POPUP_INACTIVE;
	}

	
}
