package client.graphics;

import java.awt.Graphics2D;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.HashMap;
import client.clientState.Settings;
import shared.CommonFunctions;
import shared.models.BackgroundTile;
import shared.models.Location;
/**
 * The GraphicObject for the TileBased World Background.
 * @author thorgrun
 *
 */
public class Background implements GraphicObject{

	private BackgroundTile[][] backgroundTileArray;
	public int horizontalTiles;
	public int verticalTiles;
	private Location screenFocus;
	private final static int tileSize = 16;
	private HashMap<String, BufferedImage> tiles;
	public Background(int horizontalTiles,int verticalTiles){
		this.horizontalTiles = horizontalTiles;
		this.verticalTiles = verticalTiles;
		backgroundTileArray = new BackgroundTile[horizontalTiles][verticalTiles];
		tiles = new HashMap<String, BufferedImage>();
		loadGraphicTiles();
	}
	/**
	 * Set the BackgroundTile Array.
	 * @param array
	 */
	public void setBackgroundTileSet(BackgroundTile[][] array){
		this.backgroundTileArray = array;
		if(backgroundTileArray.length>0){
			this.horizontalTiles = backgroundTileArray.length;
			this.verticalTiles = backgroundTileArray[0].length;
		}
	}
	/**
	 * Returns the BackgroundTile array.
	 * @return
	 */
	public BackgroundTile[][] getArray(){
		return backgroundTileArray;
	}
	public Background(){
		backgroundTileArray = null;
		tiles = new HashMap<String, BufferedImage>();
		loadGraphicTiles();
	}
	/**
	 * Loads the GraphicTiles.
	 */
	private void loadGraphicTiles(){
		loadTileHelper("SSSS");
		loadTileHelper("GSSS");
		loadTileHelper("SGSS");
		loadTileHelper("GGSS");
		loadTileHelper("SSGS");
		loadTileHelper("GSGS");
//		loadTileHelper("SGGS");
		loadTileHelper("GGGS");
		loadTileHelper("SSSG");
//		loadTileHelper("GSSG");
		loadTileHelper("SGSG");
		loadTileHelper("GGSG");
		loadTileHelper("SSGG");
		loadTileHelper("GSGG");
		loadTileHelper("SGGG");
		loadTileHelper("GGGG");
	}
	/**
	 * HelperMethod for loading tiles.
	 * @param tileName
	 */
	private void loadTileHelper(String tileName){
		URL fileURL = GraphicObjectFactory.getInstance().getFileURL("Graphics/Tiles/"+tileName+".png");
		tiles.put(tileName, com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT ));
	}
	/**
	 * Sets the screenFocus. Used to display the background correctly according to where the screen focus is.
	 * @param screenFocus
	 */
	public void setScreenFocus(Location screenFocus){
		this.screenFocus = screenFocus;
	}
	/**
	 * Sets one tile at a given position, with the name of the tile.
	 * @param x
	 * @param y
	 * @param textureId
	 */
	public void setTile(int x,int y,String textureId){
		backgroundTileArray[x][y] = new BackgroundTile(textureId);
		
	}
	/**
	 * Checks if a given position is within the bounds of the Background.
	 * @param x
	 * @param y
	 * @return
	 */
	public boolean withinBounds(int x,int y){
		
		if((x>CommonFunctions.drawXinPlayField(0, screenFocus.getX())&&x<CommonFunctions.drawXinPlayField(horizontalTiles*tileSize, screenFocus.getX()))
				&&(y>CommonFunctions.drawYinPlayField(0, screenFocus.getY())&&y<CommonFunctions.drawYinPlayField(verticalTiles*tileSize, screenFocus.getY()))){
			return true;
		}
		return false;
	}
	/**
	 * Draws the background relative to the position of the screen focus.
	 * Tiles that are outside the playfield area will not be drawn.
	 * TODO:Improve check.
	 */
	public void draw(Graphics2D g) {
		for(int x = 0;x<horizontalTiles;x++){
			for(int y=0;y<verticalTiles;y++){
				int drawX =CommonFunctions.drawXinPlayField(x*tileSize, screenFocus.getX());
				int drawY = CommonFunctions.drawYinPlayField(y*tileSize, screenFocus.getY());
				if((drawX<Settings.playfieldWidth+tileSize+8&&drawX>-tileSize-8)&&(drawY<Settings.playfieldHeight+tileSize+8&&drawY>-tileSize-8)){
					BackgroundTile current = backgroundTileArray[x][y];
					g.drawImage(tiles.get(current.getTextureId()), null, drawX,drawY);
				}
				
			}
			
		}

		
	}
	@Override
	public int getYLoc() {
		
		return 0;
	}
	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
}
