package client.graphics.components;

import java.awt.Button;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import client.clientState.worldState.MainWorldPanel;
import client.clientState.worldState.WorldState;
import client.graphics.GraphicObject;
import client.graphics.GraphicObjectFactory;

import shared.models.Attack;
import shared.models.AttackType;
import shared.models.SimpleCharacter.CharacterType;
/**
 * This will be the popUp where the player can choose which attack to use 
 * @author Esben Fllesdal
 *
 */
public class AttackPanel  implements GraphicObject, MouseListener{
	
	//the images has a size of 32*32
	private final int imageSizeX = 32;
	private final int imageSizeY = 32;
	
	private int xLoc = 0;
	private int yLoc = 0;
	private ArrayList<Attack> physicalAttackList = new ArrayList<Attack>();
	private ArrayList<WowButton> physicalAttackButtons = new ArrayList<WowButton>();
	private ArrayList<Attack> magicalAttackList = new ArrayList<Attack>();
	private ArrayList<WowButton> magicalAttackButtons = new ArrayList<WowButton>();
	private MainWorldPanel myWorldPanel;
	private String attackInfoString = "";
	private TextGraphicObject attackInformation;
	private String targetId;
	private int textTimer = 0;
	
	public AttackPanel(int xLoc, int yLoc, ArrayList<Attack> physicalAttackList, 
			ArrayList<Attack> magicalAttackList, MainWorldPanel myWorldPanel, String targetId, 
			CharacterType characterType){
		this.xLoc = xLoc;
		this.yLoc = yLoc;
		this.physicalAttackList = physicalAttackList;
		this.magicalAttackList = magicalAttackList;
		this.myWorldPanel = myWorldPanel;
		this.targetId = targetId;
		int locationOffset = 0;
		//creates physical attackButtons
		if(characterType == CharacterType.ENEMY_NPC){
			//can only attack an enemyNPC
			for(Attack a: physicalAttackList){
				WowButton wb = GraphicObjectFactory.getInstance().buildAttackButtonObject
									("AttackThumbs/"+a.getName().toLowerCase()+"_thumb", 
									  getXLoc()+(locationOffset*imageSizeX), getYLoc(), true);
				wb.addMouseListener(this);
				physicalAttackButtons.add(wb);
				locationOffset++;
				}	
		}
		//resets the locationOffset
		locationOffset = 0;
		//creates magical attack buttons
		for(Attack a: magicalAttackList){
			if(! a.isCanAttackFriendly() && characterType == CharacterType.ENEMY_NPC){
				//only show the attacks against an enemyNPC
			WowButton wb = GraphicObjectFactory.getInstance().buildAttackButtonObject
										("AttackThumbs/"+a.getName().toLowerCase()+"_thumb", 
										 getXLoc()+(locationOffset*imageSizeX), 
										 getYLoc()+imageSizeY, true);
			wb.addMouseListener(this);
			magicalAttackButtons.add(wb);
			locationOffset ++;
			}
			else if (a.isCanAttackFriendly() && characterType == CharacterType.FRIENDLY_NPC){
				//only show the attacks which can be used on a friendlyNPC
				WowButton wb = GraphicObjectFactory.getInstance().buildAttackButtonObject
				("AttackThumbs/"+a.getName().toLowerCase()+"_thumb", 
				 getXLoc()+(locationOffset*imageSizeX), 
				 getYLoc()+imageSizeY, true);
				wb.addMouseListener(this);
				magicalAttackButtons.add(wb);
				locationOffset ++;
			}
		}
		attackInformation = new TextGraphicObject(attackInfoString, getXLoc(), getYLoc());
	}
	
	/**
	 * Draws the attackpanel.
	 */
	public void draw(Graphics2D g) {
		//draws the physical attacks
		for(int i=0; i<physicalAttackButtons.size();i++){
			physicalAttackButtons.get(i).draw(g);
		}
		//draws the physical attacks
		for(int i=0; i<magicalAttackButtons.size();i++){
			magicalAttackButtons.get(i).draw(g);
		}
		attackInformation.draw(g);
		if(textTimer>0){
			textTimer--;
			g.setColor(Color.RED);
			g.setFont(new Font("Arial",Font.BOLD,20));
			g.drawString("Unfortunately the selected attack can not reach the target creature you have selected. Please move closer or select another target.", xLoc, yLoc-50);
		}
	}

	/**
	 * Is called when mouse is clicked.
	 * Checks if one of the attack skills was clicked.
	 */
	public void mouseClicked(MouseEvent arg0) {
		for(int i =0; i<magicalAttackButtons.size(); i++){
			if(arg0.getSource() == magicalAttackButtons.get(i)){
				if(myWorldPanel.getCharacter().useMana(magicalAttackList.get(i).getMana())){
					if(myWorldPanel.getDistanceToMouseClick(xLoc, yLoc)<magicalAttackList.get(i).getRange()){
						myWorldPanel.createQuestionPanel(magicalAttackList.get(i), targetId);
						myWorldPanel.destroyAttackPanel();
						return;
					}else{
						//TODO : Display out of range message
						textTimer = 100;
						System.out.println("Out of range.. Spell range:"+magicalAttackList.get(i).getRange());
						return;
					}
				}
				else{
					//TODO : display not enough mana
					textTimer = 100;
					System.out.println("Not enogh mana");
				}
				
			}
		}
		for(int i =0; i<physicalAttackButtons.size(); i++){
				if(arg0.getSource() == physicalAttackButtons.get(i)){
					if(myWorldPanel.getCharacter().canPerformAttack(physicalAttackList.get(i).getWeaponType())){
						if(myWorldPanel.getDistanceToMouseClick(xLoc, yLoc)<myWorldPanel.getCharacter().getAttackRange()){
							myWorldPanel.createQuestionPanel(physicalAttackList.get(i), targetId );
							myWorldPanel.destroyAttackPanel();
							return;
						}else{
							//TODO: DISPLAY OUT OF RANGE MESSAGE
							textTimer = 100;
							System.out.println("Out of range.. Weapon attack range:"+myWorldPanel.getCharacter().getAttackRange());
							return;
						}
					}
					else{
						//TODO : display not the correct weapon for this attack
						textTimer = 100;
						System.out.println("Wrong weapon for this attack");
					}
					
			}
		}
	}
	/**
	 * Updates the input of the buttons in attack panel.
	 * @param elapsedTime
	 */
	public void update(long elapsedTime){
		for(WowButton wb : magicalAttackButtons){
			wb.updateInput();
		}
		for(WowButton wb : physicalAttackButtons){
			wb.updateInput();
		}
	}

	/**
	 * Checks if mouseentered over a button.
	 */
	public void mouseEntered(MouseEvent arg0) {
		//the mouse is not hoovering over any of the buttons. Reset the text:
		attackInfoString = "";
		for(int i =0; i<magicalAttackButtons.size(); i++){
			if(arg0.getSource()==magicalAttackButtons.get(i)){
				if(! attackInfoString.equals(magicalAttackList.get(i).getName())){
					attackInfoString = magicalAttackList.get(i).getName();
					attackInformation.updateText(attackInfoString, 0);
					return;
				}
			}
		}
		for(int i =0; i<physicalAttackButtons.size(); i++){
			if(arg0.getSource()==physicalAttackButtons.get(i)){
				if(! attackInfoString.equals(physicalAttackList.get(i).getName())){
					attackInfoString = physicalAttackList.get(i).getName();
					attackInformation.updateText(attackInfoString, 0);
					return;
				}
			}
		}
	}

	/**
	 * Checks if mouse exited the area of a button.
	 */
	public void mouseExited(MouseEvent arg0) {
		attackInfoString = "";
		attackInformation.updateText(attackInfoString, 0);
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
	}

	public int getXLoc() {
		return xLoc;
	}

	public void setXLoc(int loc) {
		xLoc = loc;
	}

	public int getYLoc() {
		return yLoc;
	}

	public void setYLoc(int loc) {
		yLoc = loc;
	}

	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
}