package client.graphics.components;

import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import client.graphics.GraphicObject;
/**
 *  
 * Abstract class, inherited by other components (TextField,Buttons,etc)
 */
public abstract class SelectableComponent extends Component implements GraphicObject{

	private int xLocation = 0;
	private int yLocation = 0;
	private int height;
	private int width;
	private ArrayList<MouseListener> listOfMouseListeners;
	private boolean mouseOver; 
	
	public SelectableComponent(int xLocation, int yLocation, int width, int height, boolean mouseOver){
		this.listOfMouseListeners = new ArrayList<MouseListener>();
		this.xLocation = xLocation;
		this.yLocation = yLocation;
		this.width = width;
		this.height = height;
		this.mouseOver = mouseOver;
	}
	/**
	 * Get height of component
	 */
	public int getHeight() {
		return height;
	}
	 /**
	  * Get width of component. 
	  */
	public int getWidth() {
		return width;
	}
	/**
	 * Get X location of component.
	 * @return
	 */
	public int getXLocation() {
		return xLocation;
	}
	/**
	 * Set X location of component.
	 * @param location
	 */
	public void setXLocation(int location) {
		xLocation = location;
	}
	/**
	 * Get Y location.
	 * @return
	 */
	public int getYLocation() {
		return yLocation;
	}
	/**
	 * Set Y location.
	 * @param location
	 */
	public void setYLocation(int location) {
		yLocation = location;
	}
	
	public ArrayList<MouseListener> getListOfMouseListeners() {
		return listOfMouseListeners;
	}

	public void setListOfMouseListener(ArrayList<MouseListener> listOfMouseListeners) {
		this.listOfMouseListeners = listOfMouseListeners;
	}

	public void addMouseListener(MouseListener ml){
		listOfMouseListeners.add(ml);
	}
	
	public void removeMouseListener(MouseListener ml){
		listOfMouseListeners.remove(ml);
	}
	/**
	 * Executes MouseClicked on all mouse listeners.
	 */
	public void fireMouseClickedEvent(){
		MouseEvent me = new MouseEvent(this, 0,0L,0,0,0,0,false);
		for(MouseListener ml : listOfMouseListeners){
			ml.mouseClicked(me);
		}
	}
	
	/**
	 * Will give a message to all the mouseListeners that the mouse has entered 
	 * the object
	 */
	public void fireMouseEnteredEvent(){
		MouseEvent me = new MouseEvent(this, 0,0L,0,0,0,0,false);
		for(MouseListener ml : listOfMouseListeners){
			ml.mouseEntered(me);
		}
	}
	
	/**
	 * Will give a message to all the mouseListeners that the mouse has entered 
	 * the object
	 */
	public void fireMouseExitedEvent(){
		MouseEvent me = new MouseEvent(this, 0,0L,0,0,0,0,false);
		for(MouseListener ml : listOfMouseListeners){
			ml.mouseExited(me);
		}
	}
	
	public abstract void updateInput();
	public abstract void draw(Graphics2D g);
	/**
	 * Check if mouse button is pressed within the components area.
	 * @return
	 */
	protected boolean checkIfPressed(){
		if(client.UserInput.InputControls.getInstance().getMouseButton1Pressed()){
			if(client.UserInput.InputControls.getInstance().getMouseX() >= xLocation  &&
					client.UserInput.InputControls.getInstance().getMouseX() <= xLocation + width  &&
					client.UserInput.InputControls.getInstance().getMouseY() >= yLocation &&
					client.UserInput.InputControls.getInstance().getMouseY() <= yLocation+  height){
				return true;
			}
		}
		return false;
	}
	/**
	 * Check if the mouse enters the area of the component.
	 * @return
	 */
	protected boolean checkIfEntered(){
		if(client.UserInput.InputControls.getInstance().getMouseX() >= xLocation  &&
				client.UserInput.InputControls.getInstance().getMouseX() <= xLocation + width  &&
				client.UserInput.InputControls.getInstance().getMouseY() >= yLocation &&
				client.UserInput.InputControls.getInstance().getMouseY() <= yLocation+  height){
			return true;
		}
		return false;
	}
	/**
	 * Check if the mouse exits the area of the component.
	 * @return
	 */
	protected boolean checkIfExited(){
		if(client.UserInput.InputControls.getInstance().getMouseX() >= xLocation  &&
				client.UserInput.InputControls.getInstance().getMouseX() <= xLocation + width  &&
				client.UserInput.InputControls.getInstance().getMouseY() >= yLocation &&
				client.UserInput.InputControls.getInstance().getMouseY() <= yLocation+  height){
			return false;
		}
		return true;
	}

	public boolean isMouseOver() {
		return mouseOver;
	}

	public void setMouseOver(boolean mouseOver) {
		this.mouseOver = mouseOver;
	}
}