package client.graphics.components;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import client.graphics.GraphicObject;
/**
 * 
 * Graphical text object.
 */
public class TextGraphicObject implements GraphicObject {

	private String text;
	private int xPos;
	private int yPos;
	private Font font;
	private Color color;
	private boolean beenWordWrapped = false;
	private int availabelWidth = 0;
	private boolean doWordWrap = false;
	private boolean drawSingleLine = false;
	private ArrayList<String> linesOfText = new ArrayList<String>();
	
	private static final int standardTextSize = 12;
	
	public TextGraphicObject(String text, int xPosition, int yPosition){
		this(text, xPosition, yPosition, new Font("Arial", Font.BOLD, standardTextSize), Color.BLACK,
				0, false,false);
	}
	/**
	 * Updates the text.
	 * @param text
	 * @param i
	 */
	public void updateText(String text, int i) {
		if(i >= linesOfText.size()){
			linesOfText.add(text);
		}
		else{
			linesOfText.set(i, text);
		}
	}
	/**
	 * Updates the text.
	 * @param text
	 */
	public void updateText(String text){
		this.text = text;
	}
	public TextGraphicObject(String text, int xPosition, int yPosition, int availableWidth,
										boolean doWordWrap){
		this(text, xPosition, yPosition, new Font("Arial", Font.BOLD, standardTextSize), Color.BLACK,
				availableWidth, doWordWrap,false);
	}
	
	public TextGraphicObject(String text, int xPosition, int yPosition, Color color){
		this(text, xPosition, yPosition, new Font("Arial", Font.BOLD, standardTextSize), color,
				0, false,false);
	}
	
	public TextGraphicObject(String text, int xPosition, int yPosition, Color color,
								int availableWidth, boolean doWordWrap){
		this(text, xPosition, yPosition, new Font("Arial", Font.BOLD, standardTextSize), color,
				availableWidth, doWordWrap,false);
	}
	
	public TextGraphicObject(String text, int xPosition, int yPosition, Font font, 
									Color color){
		this(text, xPosition, yPosition, font, color, 0, false,false);
	}
	public TextGraphicObject(String text, int xPosition, int yPosition, Font font, 
			Color color,boolean drawSingleLine){
			this(text, xPosition, yPosition, font, color, 0, false,drawSingleLine);
	}
	public TextGraphicObject(String text, int xPosition, int yPosition, Font font, Color color,
									int availabelWidth, boolean doWordWrap, boolean drawSingleLine){
		this.text = text;
		this.xPos = xPosition;
		this.yPos = yPosition;
		this.font = font;
		this.color = color;
		this.availabelWidth = availabelWidth;
		this.doWordWrap = doWordWrap;
		this.drawSingleLine = drawSingleLine;
	}
	
	/**
	 * Draws the Text.
	 */
	public void draw(Graphics2D g) {
		g.setFont(font);
		if(!beenWordWrapped && doWordWrap){
			wordWrap(g.getFontMetrics(getFont()));
			beenWordWrapped = true;
		}
		if(!beenWordWrapped && !doWordWrap){
			beenWordWrapped = true;
			linesOfText.add(this.text);
		}
		Color oldColor = g.getColor();
		g.setColor(color);
		if(drawSingleLine){
			g.drawString(text, xPos, yPos);
		}else{
			for(int i=0; i<linesOfText.size();i++){
				g.drawString(linesOfText.get(i), xPos, yPos+(getFont().getSize()*i));
			}
		}
		
		g.setColor(oldColor);
	}

	public Font getFont() {
		return font;
	}
	
	/**
	 * Sets parameters which are used when calculation the word wrap
	 * @param width - the available width to where the text have to fit
	 * @param doWordWrap - true if there should be performed word wrap, 
	 * false otherwise
	 */
	public void setWordWrapParameters(int width, boolean doWordWrap){
		availabelWidth = width;
		this.doWordWrap =doWordWrap;  
	}
	
	/**
	 * executes the wordwrap
	 * @param fm - fontmetrics to the current font
	 */
	private void wordWrap(FontMetrics fm){
		//splits the text on space
		ArrayList<String> listOfWords = new ArrayList<String>();
		String tempString = "";
		for(int i =0 ; i < this.text.length(); i++){
			if(this.text.charAt(i) == ' '){
				listOfWords.add(tempString);
				tempString = "";
			}
			else {
				tempString += this.text.charAt(i);
			}
		}
		listOfWords.add(tempString);
		//must add new line symbol, where the word wrapping should happen
		//reuses the tempString
		tempString = "";
		String finishedLine = "";
		String finalText ="";
		boolean finished = false;
		while(! finished){
			if(listOfWords.size() > 0){
				tempString += listOfWords.get(0);
				tempString += " "; //adds a space
				listOfWords.remove(0);
			}
			else{
				finished = true;
				linesOfText.add(finishedLine);
				break;
			}
			if(availabelWidth - 20 > fm.stringWidth(finishedLine + tempString)){
				finishedLine += tempString;
				tempString = "";
			}
			else{
				linesOfText.add(finishedLine);
				finishedLine = tempString;
				tempString ="";
			}
		}
	}

	@Override
	public int getYLoc() {
		return yPos;
	}

	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
	
	
}
