package client.network;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.StringTokenizer;
import client.clientState.GameState;
import client.clientState.StateControl;


import shared.logger.LogSystem;
import shared.models.Attack;
import shared.models.EquipLocation;
import shared.models.Gender;
import shared.models.Item;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.PlayableClasses;
import shared.models.Player;
import shared.network.messages.ChatMessage;
import shared.network.messages.LoginMessage;
import shared.network.messages.LoginResult;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.NewCharacterFromClientMessage;
import shared.network.messages.ObjectMessage;
import shared.network.messages.PlayerVerifyMessage;
import shared.network.messages.QuestionMessage;
import shared.network.messages.RequestQuestionMessage;
import shared.network.messages.RequestZoneMessage;
import shared.network.messages.actions.ChangeZoneMessage;
import shared.network.messages.actions.EquipMessage;
import shared.network.messages.actions.MovementMessage;
import shared.network.messages.actions.SendAnswerMessage;
import shared.serverHelpers.ClientKingdomInfo;
import shared.serverHelpers.ShortCharacter;
import shared.serverHelpers.WorldServer;
/**
 * Handles messages to and from the network.
 * @author thorgrun
 *
 */
public class NetworkHandlerMain implements Runnable{
	
	private final String worldServer = "WORLDSERVER";
	private final String lobbyServer = "LOBBYSERVER";
	
	static BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
	static StringTokenizer st = new StringTokenizer("");
	private boolean running;
	private LobbyHandler lobbyHandler;
	private WorldServerHandler worldHandler;
	private ArrayList<ClientKingdomInfo> serverList;
	private boolean gotServerList;
	private int step;
	private Player player;
	private String charName;
	private String sessionId;
	private static NetworkHandlerMain singleton;
	private shared.fileHandler.Settings networkSettings;
	private NetworkHandlerMain(){
		this.player = new Player("","");
		networkSettings = new shared.fileHandler.Settings("client");
		//TODO: Get ipadress and maybe portnr from a Settings file.
		lobbyHandler = new LobbyHandler(networkSettings.getLobbyIp(),networkSettings.getLobbyPort());
		step=0;
		running = true;
		gotServerList = false;
		LogSystem.getInstance().writeClientLog("Enter username:");
		new Thread(this).start();
		
	}
	/**
	 * Returns the instance of NetworkHandlerMain
	 * @return
	 */
	public synchronized static NetworkHandlerMain getInstance(){
		if(singleton==null){
			singleton = new NetworkHandlerMain();
		}
		return singleton;
	}
	/**
	 * Is called by WorldServerHandler when a message arrives from network.
	 * @param msg
	 */
	public void getWorldServerMessage(Message msg) {
			msg.printMessage();
			StateControl.getInstance().receivedMessage(msg);		
	}
	
	private void connectToWorldServer(WorldServer server){
		worldHandler = new WorldServerHandler(server.getIpAdresse(),server.getPortNr());
	}
	private void sendLobbyMessage(Message msg){
		lobbyHandler.sendMessage(msg);
	}
	private void sendWorldServerMessage(Message msg){
		worldHandler.sendMessage(msg);
	}
	/**
	 * Sends a request message to world server.
	 * Requests a Zone object with zoneId.
	 * @param zoneId
	 */
	public void sendRequestZoneMessage(String zoneId){
		this.sendWorldServerMessage(new RequestZoneMessage(sessionId,worldServer,"I want zone:"+zoneId,MessageType.REQUEST_ZONE,zoneId));
	}
	/**
	 * Sends a movementRequest to world server.
	 * @param characterName Name of character that wants to move
	 * @param current Current position of the character
	 * @param target Target position the character want to move to.
	 */
	public void sendMovementRequest(String characterName,Location current, Location target){
		this.sendWorldServerMessage(new MovementMessage(sessionId,worldServer,"I want to move",characterName,current,target));
	}
	
	public void sendTravelTrigger(String characterName, String destinationZone, int destinationId){
		this.sendWorldServerMessage(new ChangeZoneMessage(sessionId,worldServer,"I want to change zone",characterName,destinationZone,destinationId));
	}
	
	/**
	 * Sends the answer from the client to the server
	 * @param answerId - the number the answer is in the questionPanel
	 * @param answerText - the actual answer text
	 * @see QuestionPanel
	 * @see SendAnswerMessage
	 */
	public void sendAnswerMessage(int answerId, String answerText, int timeLeft){
		this.sendWorldServerMessage(new SendAnswerMessage(sessionId,worldServer,"Here is my answer", answerId, answerText, timeLeft));
	}
	
	/**
	 *  requests a question from the server 
	 */
	public void sendRequestQuestion(Attack attack, String attackTargetId){
		this.sendWorldServerMessage(new RequestQuestionMessage(sessionId,worldServer, "I want a question", attack, attackTargetId));
	}
	/**
	 * Sends a login message to Lobby
	 * @param userName 
	 * @param password
	 */
	public void sendLobbyLoginMessage(String userName, String password){
		LoginMessage netMsg = new LoginMessage(sessionId,lobbyServer,"Login in",userName,password);
		client.network.NetworkHandlerMain.getInstance().sendLobbyMessage(netMsg);
	}
	/**
	 * Sends a chat message to world server.
	 * @param message
	 * @param chatChannel
	 */
	public void sendChatMessage(String message,String chatChannel){
		ChatMessage chatMsg = new ChatMessage(sessionId,"WorldServer",message,charName,chatChannel);
		sendWorldServerMessage(chatMsg);
	}
	/**
	 * Sends equip message to world server.
	 * @param message
	 * @param targetlocation Location you want the item
	 * @param oldlocation Current location of the item
	 * @param itemId Id of item
	 */
	public void sendEquipMessage(String message, EquipLocation targetlocation, EquipLocation oldlocation, int itemId){
		EquipMessage msg = new EquipMessage(sessionId,"WorldServer",message, "" + itemId, targetlocation, oldlocation);
		sendWorldServerMessage(msg);
	}
	/**
	 * Method to connect to the World server.
	 * @param charName Name of character you want to connect with.
	 */
	public void connectCommand(String charName){
		WorldServer server = null;
		
		for(ClientKingdomInfo kingdomInfo:serverList){
			for(ShortCharacter cha:kingdomInfo.getCharacters()){
				if(cha.getName().equals(charName)){
					server = kingdomInfo.getServerInfo();
				}
			}
			
		}
		if(server!=null){
			connectToWorldServer(server);
			this.charName = charName;
			PlayerVerifyMessage msg = new PlayerVerifyMessage(sessionId,worldServer,"I want to connect",sessionId,player.getName(),charName);
			sendWorldServerMessage(msg);
		}else{
			LogSystem.getInstance().writeClientLog("Didn't find "+charName+" in the serverList");
		}
		
	}
	
	public void sendNewPlayableCharacterRequest(String charName,PlayableClasses charClass,Gender sex,String kingdomName){
		sendLobbyMessage(new NewCharacterFromClientMessage(charName, kingdomName,sex,charClass,sessionId,"LOBBYSERVER","I want this as my new character"));
	}
	/**
	 * Connect to world server.
	 * @param charName Name of character
	 * @param server Information of server.
	 */
	public void connectCommand(String charName, WorldServer server){
		if(server!=null){
			connectToWorldServer(server);
			this.charName = charName;
			PlayerVerifyMessage msg = new PlayerVerifyMessage(sessionId,worldServer,"I want to connect",sessionId,player.getName(),charName);
			sendWorldServerMessage(msg);
		}else{
			LogSystem.getInstance().writeClientLog("Didn't find "+charName+" in the serverList");
		}
		
	}


	/**
	 * Used for debugging and early testing.
	 * Can write commands in cmd window.
	 */
	private void readCommands(){
		String userName="";
		while(running){
		String command;
			try {
				command = readString();
				switch(step){
				case 0://Entering username
					userName = command;
					player.setName(command);
					step = 1;
					LogSystem.getInstance().writeClientLog("Enter password:");
					break;
				case 1://Entering password
					LoginMessage netMsg = new LoginMessage("playerId","targetId","The message",userName,command);
					sendLobbyMessage(netMsg);
					step = 2;
					break;
				case 2:
					if(command.equals("show")){
						String show = readString();
						if(show.equals("servers")){
							printServerList();
						}
						if(show.equals("status")){
							showStatus();
						}
					}else if(command.equals("connect")){
						String charName = readString();
						connectCommand(charName);
						
					}else if(command.equals("chat")){
						String str="";
						while(st.hasMoreTokens()){
							str+=readString()+" ";
						}
						ChatMessage chatMsg = new ChatMessage(sessionId,worldServer,str,charName,"Main");
						sendWorldServerMessage(chatMsg);
					}
					break;
				}
				
			
			} catch (Exception e) {
				System.out.println("Command Read Error!");
			}
		}
	}
	
	// Read next input-token as string.
	static String readString() throws Exception {
		while (!st.hasMoreTokens()) {
			st = new StringTokenizer(stdin.readLine());
		}
		return st.nextToken();
	}
	@Override
	public void run() {
		readCommands();
	}
	public String getSessionId() {
		return sessionId;
	}
	public void setSessionId(String sessionId) {
		this.sessionId = sessionId;
	}

	
	/**
	 * Commandline helper method, display connected status
	 */
	private void showStatus(){
		StateControl.writeLog("-----Connection status-----");
		if(lobbyHandler!=null){
			LogSystem.getInstance().writeClientLog("LobbyConnected: "+lobbyHandler.isConnected());
		}else{StateControl.writeLog("LobbyConnected: false");}
	if(worldHandler!=null){
		LogSystem.getInstance().writeClientLog("WorldConnected: "+worldHandler.isConnected());
	}else{StateControl.writeLog("WorldConnected: false");}
		
	}
	
	/**
	 * Commandline helper method
	 * 
	 */
	private void printServerList(){
		StateControl.writeLog("-----SERVER LIST-----");
		StateControl.writeLog("Character name ------ KingdomName");
		for(ClientKingdomInfo kingdomInfo:serverList){
			for(ShortCharacter cha:kingdomInfo.getCharacters()){
				StateControl.writeLog(cha.getName()+" ----- "+kingdomInfo.getServerInfo().getKingdomName());
			}
			
		}
	}
	/**
	 * Received message from Lobby Server.
	 * Is called by the LobbyHandler.
	 * @param msg 
	 */
	public void getLobbyMessage(Message msg) {
		switch(msg.getMessageType()){
		case LOGIN_RESULT:{
			//THIS IS NOW THE ONLY INTERESTING PART!!
			StateControl.getInstance().receivedMessage(msg);
			LoginResult resMsg = (LoginResult)msg;
			if(resMsg.isLoginSucessfull()){
				sessionId = resMsg.getTargetId();
				LogSystem.getInstance().writeClientLog("I now have sessionId: "+sessionId);
			}
			break;
		}
		case OBJECT_MESSAGE:
			ObjectMessage objMsg = (ObjectMessage)msg;
			switch(objMsg.getObjectType()){
			case KINGDOM_INFO_LIST:
				gotServerList = true;
				serverList = (ArrayList<ClientKingdomInfo>)objMsg.getObject();
				//THIS IS NOW THE ONLY INTERESTING PART (AGAIN)!!
				StateControl.getInstance().receivedMessage(msg);
				break;
			}
			break;
		}
	}
}