package editor;


import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import editor.graphics.EditorGUI;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
import shared.network.Connection;
import shared.network.MessageHandler;
import shared.network.messages.DatabaseObjectMessage;
import shared.network.messages.KingdomMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.SessionIdMessage;

/**
 * Handles network connection and storage of objects to harddrive.
 * @author thorgrun
 *
 */
public class StorageHandler implements MessageHandler{
	private static String ENEMYNPC = "Retrieving EnemyNPC list";
	private static String FRIENDLYNPC = "Retrieving FriendlyNPC list";
	private static String STATICOBJECT = "Retrieving StaticObject list";
	private static String THEMETOPICS = "Retrieving ThemeTopics";
	private static String QUESTIONS = "Retrieving Question list";
	private static String KINGDOMNAMES = "Retrieving Kingdomnames";
	private static String ZONENAMES = "Retrieving Zonenames for templates";
	private static String KINGDOM = "Retrieving Kingdom";
	private static String ZONETEMPLATE = "Retrieving Zone Template";
	private static String PLAYERNAMES = "Retrieving Playernames";
	private static String CONNECTING = "Connecting to DB";
	private static String SAVINGTODB = "Storing in DB";
	
	private Connection conn;
	private ModelUpdater listener;
	private static StorageHandler singleton;
	private int currentNrOfRequests;
	private shared.fileHandler.Settings networkSettings;
	
	/**
	 * Returns the singleton of the StorageHandler class
	 * @return
	 */
	public static StorageHandler getInstance(){
		return singleton;
	}
	/**
	 * Initializes the StorageHandler singleton
	 * @param listener - ModelUpdater, StorageHandler receives updates from database that are forwarded to the ModelUpdater.
	 */
	public static void initialize(ModelUpdater listener){
		singleton = new StorageHandler(listener);
	}
	
	private StorageHandler(ModelUpdater listener){
		this.listener = listener;
		networkSettings = new shared.fileHandler.Settings("editor");
		currentNrOfRequests=0;
		connectToDB();
		
	
	}
	/**
	 * Opens a connection to the Database Server.
	 * If connection is established, it requests all the lists needed by the Editor.
	 */
	private void connectToDB(){
		requestSent(CONNECTING);
		conn = new Connection(networkSettings.getDatabaseIp(),networkSettings.getDatabasePort(),this);
		if(conn.isConnected()){
			
			requestAllEnemyNPC();
			requestHandled(CONNECTING);
			requestAllFriendlyNPC();
			requestAllThemeTopics();
			requestAllQuestions();
			requestAllStaticObject();
			requestAllKingdomNames();
			requestAllZoneTemplateNames();
			requestAllPlayerNames();
		}else{
			requestFailed("Could not connect to DB. Connection timed out.");
		}
	}
	
	/**
	 * Tries to reconnect to the Database Server.
	 * Does nothing if already connected.
	 */
	public void reconnectToDB(){
		if(conn!=null){
			if(!conn.isConnected()){
				connectToDB();
			}
		}
	}
	/**
	 * Sends a new EnemyNPC object to the Database for insertion.
	 * @param npc
	 */
	public void newENPC(EnemyNPC npc){
		databaseInsert(npc,ObjectType.ENEMY_NPC);
	}
	/**
	 * Sends a new FriendlyNPC object to the Database for insertion.
	 * @param npc
	 */
	public void newFNPC(FriendlyNPC npc){
		databaseInsert(npc,ObjectType.FRIENDLY_NPC);
	}
	/**
	 * Sends a new StaticObject to the database for insertion.
	 * @param obj
	 */
	public void newStaticObject(StaticObject obj){
		databaseInsert(obj,ObjectType.STATIC_OBJECT);
	}
	
	/**
	 * Sends an existing EnemyNPC to the Database for updating.
	 * @param npc
	 */
	public void updateEnemyNPC(EnemyNPC npc){
		databaseUpdate(npc,ObjectType.ENEMY_NPC);
	}
	/**
	 * Sends a existing FriendlyNPC to the Database for updating.
	 * @param npc
	 */
	public void updateFriendlyNPC(FriendlyNPC npc){
		databaseUpdate(npc,ObjectType.FRIENDLY_NPC);
	}
	/**
	 * Sends an existing StaticObject to the Database for updating.
	 * @param obj
	 */
	public void updateStaticObject(StaticObject obj){
		databaseUpdate(obj,ObjectType.STATIC_OBJECT);
	}
	/**
	 * Sends EnemyNPCid to database for deletion.
	 * @param npcId
	 */
	public void deleteEnemyNPC(int npcId){
		databaseDelete(npcId,ObjectType.ENEMY_NPC);
	}
	/**
	 * Sends FriendlyNPCid to database for deletion.
	 * @param npcId
	 */
	public void deleteFriendlyNPC(int npcId){
		databaseDelete(npcId,ObjectType.FRIENDLY_NPC);
	}
	/**
	 * Sends StaticObjectid to database for deletion.
	 * @param objId
	 */
	public void deleteStaticObject(int objId){
		databaseDelete(objId,ObjectType.STATIC_OBJECT);
	}
	/**
	 * Sends a new Question to Database for insertion.
	 * @param question
	 */
	public void newQuestion(Question question){
		databaseInsert(question,ObjectType.QUESTION);
	}
	/**
	 * Updates existing question in Database.
	 * @param question
	 */
	public void updateQuestion(Question question){
		databaseUpdate(question,ObjectType.QUESTION);
	}
	/**
	 * Deletes question from database with questionId
	 * @param questionId
	 */
	public void deleteQuestion(int questionId){
		databaseDelete(questionId,ObjectType.QUESTION);
	}
	
	/**
	 * Helper method for sending "Insert object" message to the Database.
	 * @param obj
	 * @param type
	 */
	private void databaseInsert(Object obj,ObjectType type){
		requestSent(SAVINGTODB);
		sendMessage(new DatabaseObjectMessage("Editor",
					"Database","Insert"
					,MessageType.DATABASE_INSERT,
					obj,type));
	
		
	}
	/**
	 * Helper method for sending "Update object" message to the Database.
	 * @param obj
	 * @param type
	 */
	private void databaseUpdate(Object obj,ObjectType type){
		requestSent(SAVINGTODB);
		sendMessage(new DatabaseObjectMessage("Editor",
					"Database","Update"
					,MessageType.DATABASE_ALTER,
					obj,type));
	}
	/**
	 * Helper method for sending "Delete object" message to the Database.
	 * @param obj
	 * @param type
	 */
	private void databaseDelete(Object obj, ObjectType type){
		requestSent(SAVINGTODB);
		
		sendMessage(new DatabaseObjectMessage("Editor",
					"Database","Delete"
					,MessageType.DATABASE_DELETE,
					obj,type));
	
	}
	/**
	 * Sends a request to the Database for all Theme topics.
	 */
	public void requestAllThemeTopics(){
		requestMessageToDatabase("Retrieve all theme topics", MessageType.GET_ALL_TOPICS);
		requestSent(THEMETOPICS);
	}
	/**
	 * Sends a request to the Database for all EnemyNPCs.
	 */
	public void requestAllEnemyNPC(){
		requestMessageToDatabase("Retrieve all enemy npcs", MessageType.GET_ALL_ENEMY_NPCS);
		requestSent(ENEMYNPC);
	}
	/**
	 * Sends a request to the Database for all FriendlyNPCs.
	 */
	public void requestAllFriendlyNPC(){
		requestMessageToDatabase("Retrieve all friendly NPCs",MessageType.GET_ALL_FRIENDLY_NPCS);
		requestSent(FRIENDLYNPC);
	}
	/**
	 * Sends a request to the Database for all Static objects.
	 */
	public void requestAllStaticObject(){
		requestMessageToDatabase("Retrieve all static objects",MessageType.GET_ALL_STATIC_OBJECTS);
		requestSent(STATICOBJECT);
	}
	/**
	 * Sends a request to the Database for all Questions.
	 */
	public void requestAllQuestions(){
		requestMessageToDatabase("Retreieve all questions",MessageType.GET_ALL_QUESTIONS);
		requestSent(QUESTIONS);
	}
	
	/**
	 * Sends a request to the Database for all Kingdom names.
	 */
	public void requestAllKingdomNames(){
		requestMessageToDatabase("Retrieve all kingdom names",MessageType.GET_ALL_KINGDOMNAMES);
		requestSent(KINGDOMNAMES);
	}
	/**
	 * Sends a request to the Database for all Player names.
	 */
	public void requestAllPlayerNames(){
		requestMessageToDatabase("Retrieve all Player names",MessageType.GET_ALL_PLAYERNAMES);
		requestSent(PLAYERNAMES);
	}
	/**
	 * Sends a request to the Database for all Zone Templates.
	 */
	public void requestAllZoneTemplateNames(){
		requestMessageToDatabase("Retrieve all zone names",MessageType.GET_ALL_ZONENAMES);
		requestSent(ZONENAMES);
	}
	/**
	 * Sends a request to the Database for a kingdom with the given name.
	 * @param kingdomName
	 */
	public void requestKingdom(String kingdomName){
		requestSent(KINGDOM);
		sendMessage(new SessionIdMessage("Editor","Database","Retrieve kingdom: "+kingdomName,MessageType.GET_KINGDOM,"",kingdomName));
		
	}
	/**
	 * Helper method for sending request messages to the Database.
	 * @param msg
	 * @param type
	 */
	private void requestMessageToDatabase(String msg,MessageType type){
		sendMessage(new SessionIdMessage("Editor","Database",msg,type,"",""));
	}
	/**
	 * Sends a message to the database, requesting a Zone template with the given name.
	 * @param templateName
	 */
	public void loadZoneTemplate(String templateName){
		requestSent(ZONETEMPLATE);
		sendMessage(new SessionIdMessage("Editor","Database","Retrieve kingdom: "+templateName,MessageType.GET_ZONE_TEMPLATE,"",templateName));
	}
	/**
	 * Sends a message containing the kingdom for saving in the database.
	 * @param kingdomName
	 * @param subjectName
	 * @param kingdomId
	 * @param zoneList
	 * @param zoneName
	 * @param playerAccessList
	 */
	public void saveKingdom(String kingdomName, String subjectName, int kingdomId, HashMap<String,Zone> zoneList, String zoneName, ArrayList<String> playerAccessList){
		requestSent(SAVINGTODB);
		sendMessage(new KingdomMessage(kingdomName,subjectName,kingdomId,zoneList,zoneName,playerAccessList,"Sending kingdom for saving.","Editor","DATABASE"));
	}
	/**
	 * Sends a message with a Zone template and name for saving in database.
	 * @param templateName
	 * @param zone
	 */
	public void saveTemplate(String templateName,Zone zone){
		Zone template = new Zone("-1",
				templateName,zone.getBackgroundArray(),
				zone.getRespawnPoint(),zone.getEnemyMap(),
				zone.getFriendlyMap(),zone.getStaticObjectMap(),
				new HashMap<String, TravelTrigger>());
		databaseInsert(template, ObjectType.ZONE_TEMPLATE);
	
	}
	/**
	 * Helper method for sending messages to the database.
	 * @param msg
	 */
	private void sendMessage(Message msg){
		if(conn.isConnected()){
			conn.sendMessage(msg);
		}else{
			requestFailed("lostConnection");
		}
		
	}
	/**
	 * Helper method for updating LoadDataWindow in GUI. Messages will be added as being loaded.
	 * @param message
	 */
	private void requestSent(String message){
		currentNrOfRequests+=1;
		if(currentNrOfRequests==1){//First request, show loading window
			EditorGUI.guiControls().isLoading(message);
		}else{
			EditorGUI.guiControls().addLoadMessage(message);
		}
	}
	/**
	 * Helper method for updating LoadDataWindow in GUI. 
	 * Message will be marked as Finished in GUI.
	 * @param message
	 */
	private void requestHandled(String message){
		currentNrOfRequests-=1;
		EditorGUI.guiControls().finishedLoading(message);
		if(currentNrOfRequests<=0){
			EditorGUI.guiControls().doneLoading();
		}
	}
	/**
	 * Helper method for updating LoadDataWindow in GUI.
	 * If loading fails, error message is shown and loading is aborted.
	 * @param message
	 */
	private void requestFailed(String message){
		currentNrOfRequests = 0;
		EditorGUI.guiControls().failedDBOperation(message);
	
	}

	@Override
	/**
	 * When the Connection retrieves a message, it calls this method.
	 * getMessage handles the messages sent over the network.
	 */
	public void getMessage(Message msg) {
		System.out.println("StorageHandler received: "+msg.getMessage());
		switch(msg.getMessageType()){
		case ILLEGAL_DATABASE_OPERATION:
			requestFailed(msg.getMessage());
			break;
		case SUCCESSFUL_DATABASE_OPERATION_QUESTION:
			requestAllQuestions();
			requestAllThemeTopics();
			requestHandled(SAVINGTODB);
			break;
		case SUCCESSFUL_DATABASE_OPERATION_ENEMYNPC:
			requestAllEnemyNPC();//Update the list
			requestHandled(SAVINGTODB);
			break;
		case SUCCESSFUL_DATABASE_OPERATION_FRIENDLYNPC:
			requestAllFriendlyNPC();//Update the list
			requestHandled(SAVINGTODB);
			break;
		case SUCCESSFUL_DATABASE_OPERATION_STATICOBJECT:
			requestAllStaticObject();
			requestHandled(SAVINGTODB);
			break;
		case SUCCESSFUL_DATABASE_OPERATION_KINGDOM:
			requestAllKingdomNames();//Update the List
			requestHandled(SAVINGTODB);
			SessionIdMessage sessionMsg = (SessionIdMessage)msg;
			requestKingdom(sessionMsg.getCharacterName());
			break;
		case SUCCESSFUL_DATABASE_OPERATION_ZONETEMPLATE:
			requestAllZoneTemplateNames();
			requestHandled(SAVINGTODB);
			break;
		case KINGDOM_OBJECT:
			KingdomMessage kingMsg = (KingdomMessage)msg;
			listener.updateWorldObject(kingMsg.getKingdomName(),
									kingMsg.getSubjectName(),
									kingMsg.getKingdomId(),
									kingMsg.getZoneList(),
									kingMsg.getPlayerAccessList());
			requestHandled(KINGDOM);
			break;
		case OBJECT_MESSAGE:
			ObjectMessage objMsg = (ObjectMessage)msg;
			switch(objMsg.getObjectType()){
				case ENEMY_NPC_HASHMAP:
					listener.updateAllEnemyNPCList((HashMap<String,EnemyNPC>)objMsg.getObject());
					requestHandled(ENEMYNPC);
					break;
				case FRIENDLY_NPC_HASHMAP:
					listener.updateAllFriendlyNPCList((HashMap<String,FriendlyNPC>)objMsg.getObject());
					requestHandled(FRIENDLYNPC);
					break;
				case TOPIC_LIST:
					listener.updateTopicList((ArrayList<String>)objMsg.getObject());
					requestHandled(THEMETOPICS);
					break;
				case STATIC_OBJECT_LIST:
					listener.updateAllStaticObjectList((HashMap<String,StaticObject>)objMsg.getObject());
					requestHandled(STATICOBJECT);
					break;
				case QUESTION_LIST:
					listener.updateQuestionList((ArrayList<Question>)objMsg.getObject());
					requestHandled(QUESTIONS);
					break;
				case KINGDOM_NAME_LIST:
					listener.updateKingdomNameList((ArrayList<String>)objMsg.getObject());
					requestHandled(KINGDOMNAMES);
					break;
				case ZONE_NAME_LIST:
					listener.updateZoneNameList((String[])objMsg.getObject());
					requestHandled(ZONENAMES);
					break;
				case ZONE_TEMPLATE:
					listener.addZoneFromTemplate((Zone)objMsg.getObject());
					requestHandled(ZONETEMPLATE);
					break;
				case PLAYER_NAME_LIST:
					listener.updatePlayerNameList((ArrayList<String>)objMsg.getObject());
					requestHandled(PLAYERNAMES);
					break;
			}
			break;
		}
			
	}
	/**
	 * Stores a kingdom on the Harddrive as a serializable object.
	 * @param kingdomName
	 * @param subjectName
	 * @param kingdomId
	 * @param zoneList
	 * @param zoneName
	 * @param playerAccessList
	 * @return
	 */
	public boolean saveWorldToHarddrive(String kingdomName,
											String subjectName,
											int kingdomId,
											HashMap<String,Zone> zoneList,
											String zoneName, 
											ArrayList<String> playerAccessList){
		//Puts the kingdom in a serializable message that can be dumped to harddrive
		KingdomMessage message = new KingdomMessage(kingdomName,subjectName,kingdomId,zoneList,zoneName,playerAccessList,"Saving kingdom to Harddrive.","Editor","Harddrive");
		FileOutputStream f_out;
		try {
			f_out = new FileOutputStream ("Resources/KingdomBackups/"+kingdomName+".WoW");
		
			ObjectOutputStream obj_out = new
		    ObjectOutputStream (f_out);
	
			obj_out.writeObject(message);
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			System.out.println("FileNotFoundException:"+e.getMessage());
			return false;
		}catch(IOException e){
			System.out.println("IOException:"+e.getMessage());
			return false;
		}
		return true;
	}
	/**
	 * Loads kingdom from a file. 
	 * @param filePath Path to the kingdom file.
	 */
	public void loadWorldFromHarddrive(String filePath){
		// Read from disk using FileInputStream.
		requestSent(KINGDOM);
		FileInputStream f_in;
		try {
			f_in = new FileInputStream(filePath);
	
		
			ObjectInputStream obj_in = new ObjectInputStream (f_in);
		
			Object obj = obj_in.readObject ();
		
		
			if (obj instanceof KingdomMessage)
			{
				KingdomMessage kingMsg = (KingdomMessage) obj;
				listener.updateWorldObject(kingMsg.getKingdomName(),
						kingMsg.getSubjectName(),
						kingMsg.getKingdomId(),
						kingMsg.getZoneList(),
						kingMsg.getPlayerAccessList());
				requestHandled(KINGDOM);
			}else{
				requestFailed("Wrong fileformat.");
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			System.out.println("FileNotFoundException:"+e.getMessage());
		}catch(ClassNotFoundException e){
			System.out.println("ClassNotFoundException:"+e.getMessage());
		}catch(IOException e){
			System.out.println("IOException:"+e.getMessage());
		}
	}
	
	@Override
	/**
	 * Called by connection when the connection is lost.
	 * Aborts loading of data.
	 */
	public void lostConnection() {
		requestSent("");
		requestFailed("Database connection lost.");
	}
	
	
	
}
