package editor.graphics;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import editor.EditorMain;
import editor.graphics.editorPanels.mapEditorPanels.TileIcon;

/**
 * Used for loading images from harddrive.
 * @author thorgrun
 *
 */
public final class ImageFactory {
	private static ImageFactory singleton;
	private static String ENEMY_IMAGE_FOLDER = "Resources/Graphics/WorldObjects/";
	private static String FRIENDLY_IMAGE_FOLDER = "Resources/Graphics/WorldObjects/";
	private static String STATIC_OBJECT_IMAGE_FOLDER="Resources/Graphics/WorldObjects/";
	//Buffered images
	private Image respawnPoint;
	private Image notFound;
	private HashMap<String, Image> tiles;
	/**
	 * Private constructor.
	 * Initializes list of tiles and loads from harddrive.
	 * Loads notFound image and respawnPoint image.
	 */
	private ImageFactory(){

		tiles = new HashMap<String, Image>();
		notFound = getImage("Resources/Graphics/noimage.png");
		respawnPoint = getImage("Resources/Graphics/respawnpoint.png");
		loadGraphicTiles();
	}
	/**
	 * Returns instance of ImageFactory as singleton.
	 * @return
	 */
	public synchronized static ImageFactory getInstance(){
		if(singleton == null){
			singleton = new ImageFactory();
		}
		return singleton;
	}
	/**
	 * Return respawnPoint image used in zones.
	 * @return
	 */
	public Image getRespawnImage(){
		return respawnPoint;
	}
	/**
	 * Returns image of EnemyNPC by name
	 * @param enemyName
	 * @return
	 */
	public Image getEnemyImage(String enemyName){
		String imageName = EditorMain.getControls().getEnemyNPC(enemyName).getImagePath();
		return getImage(ENEMY_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Returns image of FriendlyNPC by name
	 * @param friendlyName
	 * @return
	 */
	public Image getFriendlyImage(String friendlyName){
		String imageName = EditorMain.getControls().getFriendlyNPC(friendlyName).getImagePath();
		return getImage(FRIENDLY_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Returns StaticObject image by name.
	 * @param staticName
	 * @return
	 */
	public Image getStaticObjectImage(String staticName){
		String imageName = EditorMain.getControls().getStaticObject(staticName).getTextureId();
		return getImage(STATIC_OBJECT_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Get enemy image by name from enemy image folder.
	 * @param imageName
	 * @return
	 */
	public Image getEnemyImageByImageName(String imageName){
		return getImage(ENEMY_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Get friendly image by name from friendly image folder.
	 * @param imageName
	 * @return
	 */
	public Image getFriendlyImageByImageName(String imageName){
		return getImage(FRIENDLY_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Get static object by name from static object image folder.
	 * @param imageName
	 * @return
	 */
	public Image getStaticObjectImageByImageName(String imageName){
		return getImage(STATIC_OBJECT_IMAGE_FOLDER+imageName+".png");
	}
	/**
	 * Returns a background tile by name.
	 * @param tilename
	 * @return
	 */
	public Image getTile(String tilename){
		return tiles.get(tilename);
	}
	
	/**
	 * Returns a list of background tiles.
	 * @return
	 */
	public ArrayList<TileIcon> getAllBackgroundTiles(){
		ArrayList<TileIcon> imgList = new ArrayList<TileIcon>();
		for(String str : tiles.keySet()){
			imgList.add(new TileIcon(tiles.get(str),str));
		}
		return imgList;
	}
	/**
	 * Returns a list of main background tiles.
	 * @return
	 */
	public ArrayList<TileIcon> getBackgroundTiles(){
		ArrayList<TileIcon> imgList = new ArrayList<TileIcon>();
		imgList.add(new TileIcon(tiles.get("SSSS"),"SSSS"));
		imgList.add(new TileIcon(tiles.get("GGGG"),"GGGG"));
		return imgList;
	}
	/**
	 * Loads the GraphicTiles.
	 */
	private void loadGraphicTiles(){
		loadTileHelper("SSSS");
		loadTileHelper("GSSS");
		loadTileHelper("SGSS");
		loadTileHelper("GGSS");
		loadTileHelper("SSGS");
		loadTileHelper("GSGS");
//		loadTileHelper("SGGS");
		loadTileHelper("GGGS");
		loadTileHelper("SSSG");
	//	loadTileHelper("GSSG");
		loadTileHelper("SGSG");
		loadTileHelper("GGSG");
		loadTileHelper("SSGG");
		loadTileHelper("GSGG");
		loadTileHelper("SGGG");
		loadTileHelper("GGGG");
	}
	/**
	 * HelperMethod for loading tiles.
	 * @param tileName
	 */
	private void loadTileHelper(String tileName){
		String filename = "Resources/Graphics/Tiles/"+tileName+".png";
		tiles.put(tileName, getImage(filename));
	}
	
	public String copyImageToWorldObjectFolder(String filePath){
		BufferedImage img = null;
		String imageName=null;
		try {
			File imageFile = new File(filePath);
			int lastDot = imageFile.getName().lastIndexOf('.');
			
			if (0 < lastDot && lastDot <= imageFile.getName().length() - 2 ) {
				imageName = imageFile.getName().substring(0, lastDot);
			} 
			File worldObjectDirectory = new File("Resources/Graphics/WorldObjects/");
			//Check if the image is already in world objects, not read and copy if it is.
			if(!imageFile.getParent().equals(worldObjectDirectory.getAbsolutePath())){
				img = ImageIO.read(imageFile);
			    if(img!=null){
					File outputfile = new File("Resources/Graphics/WorldObjects/"+imageName+".png");
					ImageIO.write(img, "png", outputfile);
					System.out.println("Successfully copied image "+imageName+".png");
			    }else{
			    	System.err.println("File was not an image");
			    }
			}
		} catch (IOException e) {
			System.err.println("Error occured while moving image");
			System.out.println(e.getStackTrace());
		}
		return imageName;
	}

	/**
	 * Helpermethod for loading images from harddrive
	 * @param filePath
	 * @return
	 */
	private Image getImage(String filePath){
	      Image img =  new ImageIcon(filePath).getImage();
	      if(img.getHeight(null)<0){
	        	  System.err.println("Can't find file '" + filePath + "'");
		          return notFound;
	      }
	      return img;
	}
/*
 
 * Old method for loading images from harddrive. Uses getResource
    	  Image img = null;
	     
	      URL url = getClass().getResource(filePath);
	      if ( url == null ){
	          System.err.println("Can't find file '" + filePath + "'");
	          return notFound;
	      }
	      img = java.awt.Toolkit.getDefaultToolkit().getImage(url);
	      return img;
 */
}

