package editor.graphics.editorPanels.mapEditorPanels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JPanel;

import editor.graphics.ImageFactory;
import shared.models.BackgroundTile;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Location;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.WorldObject;
import shared.models.Zone;


/**
 * Displays the graphics of the current Kingdom.
 * Takes input from Mouse actions.
 * @author thorgrun
 *
 */
public class MapGraphics extends JPanel implements MouseListener, MouseMotionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	private final static int tileSize = 16;
	
	
	private Zone drawZone;
	private int playfieldWidth;
	private int playfieldHeight;
	private Location screenFocus;
	private WorldObject selectedWorldObject;
	private boolean isDragging;
	private int mouseX;
	private int mouseY;
	private String paintTileName;
	private boolean isPaintingTiles;
	private boolean respawnSelected;
	private boolean leftMouseButtonClicked=false;
	private ObjectSelectedListener listener;
	public MapGraphics(ObjectSelectedListener listener,int width, int height){
		this.listener = listener;
		playfieldWidth = width;
		playfieldHeight = height;
		setBackground(Color.WHITE);
		isDragging = false;
		screenFocus = new Location(playfieldWidth/2,playfieldHeight/2);
		addMouseListener(this);
		addMouseMotionListener(this);
		setPreferredSize(new Dimension(playfieldWidth,playfieldHeight));
	}
	@Override public void paintComponent(Graphics g){
		super.paintComponent(g);
		if(drawZone!=null){
			drawBackground(g);
			//Draw respawn point
			if(drawZone.getRespawnPoint()!=null){
				int drawX= drawZone.getRespawnPoint().getX()-10 - (screenFocus.getX()-playfieldWidth/2);
				int drawY = drawZone.getRespawnPoint().getY()-10 - (screenFocus.getY()-playfieldHeight/2); 
			
				g.drawImage(ImageFactory.getInstance().getRespawnImage(),
						drawX,drawY,this);
			}
			
			
			for(EnemyNPC npc :drawZone.getEnemyList()){
				drawWorldObject(npc, g);
			}
			for(FriendlyNPC npc : drawZone.getFriendlyList()){
				drawWorldObject(npc,g);
			}
			for(StaticObject obj : drawZone.getStaticObjectList()){
				drawWorldObject(obj,g);
			}
			for(TravelTrigger trigger : drawZone.getTravelTriggerList()){
				int drawX= drawXinPlayField(trigger.getCurrentLocation().getX(),trigger.getImageWidth());
				int drawY = drawYinPlayField(trigger.getCurrentLocation().getY(),trigger.getImageHeight());
				g.setColor(Color.blue);
				g.fillRect(drawX, drawY, trigger.getImageWidth(),trigger.getImageHeight());
				g.setColor(Color.black);
			}
		
		}
		if(selectedWorldObject!=null||respawnSelected){
			g.setColor(Color.green);
			
			
			if(isDragging){
				//Draw rectangle while moving cursor
				if(selectedWorldObject!=null){
					g.drawRect(mouseX-selectedWorldObject.getImageWidth()/2,mouseY-selectedWorldObject.getImageHeight()/2,
							selectedWorldObject.getImageWidth(), 
							selectedWorldObject.getImageHeight());
				}else{
					g.drawRect(mouseX-10,mouseY-10,
							20, 
							20);
				}
				
				repaint();
			}else{
				//Draw rectangle around object
				if(selectedWorldObject!=null){
					g.drawRect(
							drawXinPlayField(selectedWorldObject.getCurrentLocation().getX(),selectedWorldObject.getImageWidth()),
							drawYinPlayField(selectedWorldObject.getCurrentLocation().getY(),selectedWorldObject.getImageHeight()), 
							selectedWorldObject.getImageWidth(), 
							selectedWorldObject.getImageHeight());
				}else{
					int drawX= drawZone.getRespawnPoint().getX()-10 - (screenFocus.getX()-playfieldWidth/2);
					int drawY = drawZone.getRespawnPoint().getY()-10 - (screenFocus.getY()-playfieldHeight/2); 
					g.drawRect(drawX,drawY,20,20);
				}
			
			}
			g.setColor(Color.white);
		}
		
		
		
		//g.drawImage(getImage("/Resources/Graphics/WorldObjects/testTree.png"),5,5,this);
		
		
		
		
	}
	/**
	 * Enables painting of backgroundtiles.
	 * Leftclicking mouse will then paint tile with given name.
	 * @param tileName
	 */
	public void enablePainting(String tileName){
		this.paintTileName = tileName;
		isPaintingTiles = true;
		
	}
	/**
	 * Disables the background painting.
	 */
	public void disableTilePainting(){
		isPaintingTiles = false;
	}
	/**
	 * Returns the current focus of the graphics. Where in the zone the screen focuses.
	 * @return
	 */
	public Location getScreenFocus(){
		return screenFocus;
	}
	/**
	 * Sets the screenfocus.
	 * @param newFocus
	 */
	public void setScreenFocus(Location newFocus){
		screenFocus.setX(newFocus.getXDouble());
		screenFocus.setY(newFocus.getYDouble());
	}
	/**
	 * Selects an enemyNPC, displays a green rectangel around it.
	 * @param id
	 */
	public void selectEnemyNPC(String id){
		setSelectedWorldObject(drawZone.getEnemyNPC(id));
	}
	/**
	 * Selects a friendlyNPC, displays a green rectangel around it.
	 * @param id
	 */
	public void selectFriendlyNPC(String id){
		setSelectedWorldObject(drawZone.getFriendlyNPC(id));
	}
	/**
	 * Selects a StaticObject, displays a green rectangel around it.
	 * @param id
	 */
	public void selectStaticObject(String id){
		setSelectedWorldObject(drawZone.getStaticObject(id));
	}
	/**
	 * Selects a TravelTrigger,displays a green rectangel around it.
	 * @param id
	 */
	public void selectTravelTrigger(String id){
		setSelectedWorldObject(drawZone.getTravelTrigger(id));
	}
	/**
	 * Helper method for selecting a worldObject.
	 * @param obj
	 */
	private void setSelectedWorldObject(WorldObject obj){
		this.selectedWorldObject = obj;
		respawnSelected = false;
		repaint();
	}
	
	public void removeSelectionOfObject(){
		this.selectedWorldObject = null;
		repaint();
	}
	/**
	 * Helpermethod for drawing a worldobject in the panel.
	 * @param obj
	 * @param g
	 */
	private void drawWorldObject(WorldObject obj, Graphics g){
		Image img = null;
		if(obj instanceof StaticObject){
			img = ImageFactory.getInstance().getStaticObjectImage(((StaticObject)obj).getName());
		}else if(obj instanceof EnemyNPC){
			img = ImageFactory.getInstance().getEnemyImage(((EnemyNPC)obj).getName());
		}else if(obj instanceof FriendlyNPC){
			img = ImageFactory.getInstance().getFriendlyImage(((FriendlyNPC)obj).getName());
		}
		obj.setImageHeight(img.getHeight(this));
		obj.setImageWidth(img.getWidth(this));
		g.drawImage(img,
				drawXinPlayField(obj.getCurrentLocation().getX(),obj.getImageWidth()),
				drawYinPlayField(obj.getCurrentLocation().getY(),obj.getImageHeight()),
				this);
	}
	
	/**
	 * Helpermethod for drawing the background of the currently selected zone.
	 * @param g
	 */
	private void drawBackground(Graphics g){
		for(int x = 0;x<drawZone.getHorizontalSize();x++){
			for(int y=0;y<drawZone.getVerticalSize();y++){
				int drawX =drawXinPlayField(x*tileSize);
				int drawY =drawYinPlayField(y*tileSize);
				if((drawX<playfieldWidth+tileSize+8&&drawX>-tileSize-8)&&(drawY<playfieldHeight+tileSize+8&&drawY>-tileSize-8)){
					BackgroundTile current = drawZone.getBackgroundArray()[x][y];
					g.drawImage(ImageFactory.getInstance().getTile(current.getTextureId()), drawX,drawY,this);
				}
				
			}
			
		}
	}
	/**
	 * Helpermethod for painting tiles in the background.
	 * @param gameX
	 * @param gameY
	 */
	private void paintTileAt(int gameX,int gameY){
		int tilePositionX = gameX/tileSize;
		int tilePositionY = gameY/tileSize;
	
		
		if(!drawZone.getBackgroundArray()[tilePositionX][tilePositionY].getTextureId().equals(paintTileName)){
			drawZone.setTileWithCascade(tilePositionX, tilePositionY, new BackgroundTile(paintTileName));
			repaint();
		}
		
	}
	/**
	 * Method for setting selected zone. Which zone that will be drawn.
	 * @param drawZone
	 */
	public void setDrawZone(Zone drawZone){
		this.drawZone = drawZone;
		this.selectedWorldObject = null;
		this.repaint();
	}
	
	/**
	 * Helper method for getting position in panel for drawing an object with given playfield position.
	 * @param playFieldPositionX
	 * @param imageWidth
	 * @return
	 */
	private int drawXinPlayField(int playFieldPositionX,int imageWidth){
		return (playFieldPositionX-imageWidth/2) - (screenFocus.getX()-playfieldWidth/2);
	}
	/**
	 * Helper method for getting position in panel for drawing an object with given playfield position.
	 * @param playFieldPositionY
	 * @param imageHeight
	 * @return
	 */
	private int drawYinPlayField(int playFieldPositionY,int imageHeight){
		return (playFieldPositionY-imageHeight) - (screenFocus.getY()-playfieldHeight/2);
	}
	/**
	 * Helper method for getting position in panel for drawing an object with given playfield position.
	 * @param playFieldPositionX
	 * @return
	 */
	private int drawXinPlayField(int playFieldPositionX){
		return playFieldPositionX - (screenFocus.getX()-playfieldWidth/2);
	}
	/**
	 * Helper method for getting position in panel for drawing an object with given playfield position.
	 * @param playFieldPositionY
	 * @return
	 */
	private int drawYinPlayField(int playFieldPositionY){
		return playFieldPositionY - (screenFocus.getY()-playfieldHeight/2);
	}
	/**
	 * Checks that the mouse cursor is within the playfield.
	 * @param mouseX
	 * @param mouseY
	 * @return
	 */
	private boolean withinBoundaries(int mouseX,int mouseY){
		int x = getXinPlayField(mouseX);
		int y = getYinPlayField(mouseY);
		return (y>0&&x>0)&&(x<drawZone.getHorizontalSize()*tileSize&&y<drawZone.getVerticalSize()*tileSize);
	}
	/**
	 * Returns the playfield position of mouse cursor.
	 * @param mouseX
	 * @return
	 */
	public int getXinPlayField(int mouseX){
		return mouseX + (screenFocus.getX()-playfieldWidth/2);
	}
	/**
	 *Returns the playfield position of mouse cursor.
	 */
	public int getYinPlayField(int mouseY){
		return mouseY + (screenFocus.getY()-playfieldHeight/2);
	}
	/**
	 * Check if a given object is at position of mouse cursor.
	 * Used for selecting objects in the zone.
	 * @param object
	 * @param mouseX
	 * @param mouseY
	 * @return
	 */
	private boolean checkIfObjectIsAtPosition(WorldObject object, int mouseX,int mouseY){
		int objectX = drawXinPlayField(object.getCurrentLocation().getX(), object.getImageWidth());
		int objectY = drawYinPlayField(object.getCurrentLocation().getY(),object.getImageHeight());
		
		if((mouseX<objectX+object.getImageWidth()&&mouseX>objectX)&&(mouseY<objectY+object.getImageHeight()&&mouseY>objectY)){
			return true;
		}
		return false;
	}
	/**
	 * Returns the worldObject that was selected by user.
	 * @param mouseX
	 * @param mouseY
	 * @return
	 */
	private WorldObject getWorldObjectClicked(int mouseX, int mouseY){
		for(FriendlyNPC npc : drawZone.getFriendlyList()){
			if(checkIfObjectIsAtPosition(npc, mouseX, mouseY)){
				return npc;
			}
		}
		for(EnemyNPC npc : drawZone.getEnemyList()){
			if(checkIfObjectIsAtPosition(npc, mouseX, mouseY)){
				return npc;
			}
		}
		for(StaticObject obj : drawZone.getStaticObjectList()){
			if(checkIfObjectIsAtPosition(obj, mouseX, mouseY)){
				return obj;
			}
		}
		for(TravelTrigger trigger: drawZone.getTravelTriggerList()){
			if(checkIfObjectIsAtPosition(trigger,mouseX,mouseY)){
				return trigger;
			}
		}
		return null;
	}
	/**
	 * Checks if the user selected the respawnpoint.
	 * @param mouseX
	 * @param mouseY
	 * @return
	 */
	private boolean respawnClicked(int mouseX,int mouseY){
		int drawX= drawZone.getRespawnPoint().getX()-10 - (screenFocus.getX()-playfieldWidth/2);
		int drawY = drawZone.getRespawnPoint().getY()-10 - (screenFocus.getY()-playfieldHeight/2); 
		
		if((mouseX<drawX+20&&mouseX>drawX)&&(mouseY<drawY+20&&mouseY>drawY)){
			return true;
		}
		return false;
	}

	
	@Override
	public void mouseClicked(MouseEvent e) {
		
		
	}
	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void mousePressed(MouseEvent e) {
		if(isPaintingTiles&&e.getButton()==MouseEvent.BUTTON1){
			leftMouseButtonClicked = true;
			if(withinBoundaries(e.getX(), e.getY())){
				paintTileAt(getXinPlayField(e.getX()), getYinPlayField(e.getY()));
			}
		}else if(e.getButton()==MouseEvent.BUTTON1){
			leftMouseButtonClicked = true;
			if(clickedOnSelectedObject(getXinPlayField(e.getX()),getYinPlayField(e.getY()))||(respawnClicked(e.getX(),e.getY())&&respawnSelected)){
				isDragging = true;
				mouseX = e.getX();
				mouseY = e.getY();
				repaint();
			}
			if(!isDragging){
				WorldObject clickedobject = getWorldObjectClicked(e.getX(),e.getY());
				if(clickedobject instanceof EnemyNPC){
					listener.enemyNPCSelected(drawZone.getZoneName(),((EnemyNPC)clickedobject).getEnemyNPCId()+"");
				}else if(clickedobject instanceof FriendlyNPC){
					listener.friendlyNPCSelected(drawZone.getZoneName(),((FriendlyNPC)clickedobject).getFriendlyNPCId()+"");
				}else if(clickedobject instanceof StaticObject){
					listener.staticObjectSelected(drawZone.getZoneName(),((StaticObject)clickedobject).getStaticObjectId()+"");
				}else if(clickedobject instanceof TravelTrigger){
					listener.travelTriggerSelected(drawZone.getZoneName(),((TravelTrigger)clickedobject).getTravelTriggerId()+"");
				}else{
					if(respawnClicked(e.getX(),e.getY())){
						respawnSelected = true;
						selectedWorldObject = null;
					}
				}
				repaint();
			}
			
		
		}else{
			leftMouseButtonClicked = false;
		}
	
		
	}
	private boolean clickedOnSelectedObject(int x, int y){
		WorldObject obj = selectedWorldObject;
		if(obj!=null){
			double leftMostObjectPosX = obj.getCurrentLocation().getX()-obj.getImageWidth()/2;
			double rightMostObjectPosX = obj.getCurrentLocation().getX()+obj.getImageWidth()/2;
			double topY = obj.getCurrentLocation().getY();
			double bottomY = obj.getCurrentLocation().getY()-obj.getImageHeight();
			if(x>leftMostObjectPosX&&x<rightMostObjectPosX){
				if(y>bottomY&&y<topY){
					return true;
				}
			}
		}
		return false;
	}
	@Override
	public void mouseReleased(MouseEvent e) {
		if(e.getButton()==MouseEvent.BUTTON3){
			if(withinBoundaries(e.getX(),e.getY())){
				screenFocus= new Location(getXinPlayField(e.getX()),getYinPlayField(e.getY()));
				this.repaint();
			}
			
		}else if(e.getButton()==MouseEvent.BUTTON1){
			if(withinBoundaries(e.getX(),e.getY())){
				if(isDragging){
					if(selectedWorldObject!=null){
						selectedWorldObject.setCurrentLocation(new Location(getXinPlayField(e.getX()),getYinPlayField(e.getY()+selectedWorldObject.getImageHeight()/2)));
					}else{
						drawZone.getRespawnPoint().setX(getXinPlayField(e.getX()));
						drawZone.getRespawnPoint().setY(getYinPlayField(e.getY()));
					}
					
				}
			}
		}
		isDragging = false;
		repaint();
		
	}
	@Override
	public void mouseDragged(MouseEvent e) {
		if(isPaintingTiles&&leftMouseButtonClicked){
			if(withinBoundaries(e.getX(), e.getY())){
				paintTileAt(getXinPlayField(e.getX()), getYinPlayField(e.getY()));
			}
		}else{
			mouseX = e.getX();
			mouseY = e.getY();
		}
		
		
	}
	@Override
	public void mouseMoved(MouseEvent e) {
		
		
	}
}
