package editor.models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import editor.StorageHandler;
import editor.models.changeListeners.AccessChangedListener;
import editor.models.changeListeners.WorldModelChangedListener;

import shared.models.BackgroundTile;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
/**
 * Contains the Kingdom objects. All variables connected to the kingdom.
 * 
 * @author thorgrun
 *
 */
public class WorldModel {

	private HashMap<String, Zone> kingdom;
	private String kingdomName="Kingdom of TEST";
	private String kingdomSubject="History of Thor";
	private int kingdomId=0;
	private ArrayList<String> playerAccessList;
	private ArrayList<String> playerNames;
	private WorldModelChangedListener worldListener;
	private AccessChangedListener accessListener;
	private String selectedZone;
	private int idDecrement;
	/**
	 * Constructor of WorldModel.
	 * Adds WorldModelChangedListener.
	 * Initializes empty kingdom and playerAccessList.
	 * @param listener
	 */
	public WorldModel(WorldModelChangedListener listener){
		this.worldListener = listener;
		kingdom = new HashMap<String,Zone>();
		playerAccessList = new ArrayList<String>();
		idDecrement = 0;
	}
	/**
	 * Updates the world object with a new kingdom. 
	 * Notifies listener of changes.
	 * @param kingdomName
	 * @param kingdomSubject
	 * @param kingdomId
	 * @param kingdom
	 * @param playerAccessList
	 */
	public void setWorld(String kingdomName,String kingdomSubject,int kingdomId,HashMap<String, Zone> kingdom,ArrayList<String> playerAccessList){
		//ClearGuI
		worldListener.newWorld(kingdomName);
		
		this.kingdomName = kingdomName;
		this.kingdomId = kingdomId;
		this.kingdomSubject = kingdomSubject;
		this.playerAccessList = playerAccessList;
		accessListener.updatePlayerAccessList(playerAccessList);
		if(kingdom==null){
			this.kingdom = new HashMap<String,Zone>();
		}else{
			this.kingdom = kingdom;
		}
		for(String zone : this.kingdom.keySet()){
			worldListener.newZone(this.kingdom.get(zone));
		}
		
		
	}
	/**
	 * Sets AccessChangedListener.
	 * @param listener
	 */
	public void setAccessListener(AccessChangedListener listener){
		this.accessListener = listener;
	}
	/**
	 * Updates list of list of player names and notifies accesslistener of changes.
	 * @param playerNames
	 */
	public void updatePlayerNameList(ArrayList<String> playerNames){
		this.playerNames = playerNames;
		accessListener.updatePlayerNames(this.playerNames);
		accessListener.updatePlayerAccessList(playerAccessList);
	}
	/**
	 * Give access to kingdom for player.
	 * @param playerName
	 */
	public void addPlayerAccess(String playerName){
		playerAccessList.add(playerName);
		accessListener.addPlayer(playerName);
	}
	/**
	 * Remove access to kingdom for player.
	 * @param playerName
	 */
	public void removePlayerAccess(String playerName){
		playerAccessList.remove(playerName);
		accessListener.removePlayer(playerName);
	}
	/**
	 * Adds a zone from template.
	 * Resets the Id of objects in zone, to avoid using same objects in database for several kingdoms.
	 * @param zoneTemplate
	 */
	public void addZoneFromTemplate(Zone zoneTemplate){
		for(EnemyNPC eNpc : zoneTemplate.getEnemyList()){
			idDecrement--;
			zoneTemplate.removeEnemyNPC(eNpc.getEnemyNPCId()+"");
			eNpc.setEnemyNPCId(idDecrement);
			zoneTemplate.addEnemyNPC(eNpc.getEnemyNPCId()+"", eNpc);
		}
		for(FriendlyNPC fNpc : zoneTemplate.getFriendlyList()){
			idDecrement--;
			zoneTemplate.removeFriendlyNPC(fNpc.getFriendlyNPCId()+"");
			fNpc.setFriendlyNPCId(idDecrement);
			zoneTemplate.addFriendlyNPC(fNpc.getFriendlyNPCId()+"", fNpc);
		}
		for(StaticObject obj : zoneTemplate.getStaticObjectList()){
			idDecrement--;
			zoneTemplate.removeStaticObject(obj.getStaticObjectId()+"");
			obj.setStaticObjectId(idDecrement);
			zoneTemplate.addStaticObject(obj.getStaticObjectId()+"", obj);
		}
		idDecrement--;
		zoneTemplate.setZoneId(idDecrement+"");
		kingdom.put(zoneTemplate.getZoneName(), zoneTemplate);
		worldListener.newZone(zoneTemplate);
		
	}
	/**
	 * Creates a new zone in kingdom and notifies WorldListener.
	 * @param newZone
	 * @return
	 */
	public boolean createNewZone(Zone newZone){
		if(kingdom.get(newZone.getZoneName())==null){
			//Set all backgroundtiles to default tiles
			for(int x = 0;x<newZone.getHorizontalSize();x++){
				for(int y = 0;y<newZone.getVerticalSize();y++){
					newZone.setTile(x, y, new BackgroundTile("GGGG"));
				}
			}
			kingdom.put(newZone.getZoneName(), newZone);
			worldListener.newZone(newZone);
			return true;
		}
		return false;
	}
	/**
	 * Returns zone by name.
	 * @param zoneName
	 * @return
	 */
	public Zone getZone(String zoneName){
		return kingdom.get(zoneName);
	}
	/**
	 * Selects a zone.
	 * Notifies WorldListener of currentzone.
	 * @param zoneName
	 */
	public void selectZone(String zoneName){
		selectedZone=zoneName;
		worldListener.currentZone(kingdom.get(zoneName));
	}
	/**
	 * Adds EnemyNPC to zone.
	 * Creates a new id for EnemyNPC
	 * @param zoneName
	 * @param npc
	 */
	public void addEnemyNPCToZone(String zoneName,EnemyNPC npc){
		idDecrement--;
		npc.setEnemyNPCId(idDecrement);
		kingdom.get(zoneName).addEnemyNPC(npc.getEnemyNPCId()+"", npc);
		worldListener.newEnemyNPC(zoneName, npc);
	}
	/**
	 * Adds FriendlyNPC to zone.
	 * Creates a new id for FriendlyNPC
	 * @param zoneName
	 * @param npc
	 */
	public void addFriendlyNPCToZone(String zoneName,FriendlyNPC npc){
		idDecrement--;
		npc.setFriendlyNPCId(idDecrement);
		kingdom.get(zoneName).addFriendlyNPC(npc.getFriendlyNPCId()+"", npc);
		worldListener.newFriendlyNPC(zoneName, npc);
	}
	/**
	 * Adds StaticObject to zone.
	 * Creates a new id for StaticObject.
	 * @param zoneName
	 * @param obj
	 */
	public void addStaticObjectToZone(String zoneName,StaticObject obj){
		idDecrement--;
		obj.setStaticObjectId(idDecrement);
		kingdom.get(zoneName).addStaticObject(obj.getStaticObjectId()+"", obj);
		worldListener.newStaticObject(zoneName, obj);
	}
	/**
	 * Adds a TravelTrigger to zone.
	 * Checks if unique.
	 * Then creates new id for trigger.
	 * @param zoneName
	 * @param trigger
	 * @return
	 */
	public boolean addTravelTriggerToZone(String zoneName, TravelTrigger trigger){
		if(!triggerExists(trigger.getName())){
			idDecrement--;
			trigger.setTravelTriggerId(idDecrement);
			kingdom.get(zoneName).addTravelTrigger(trigger.getTravelTriggerId()+"", trigger);
			worldListener.newTravelTrigger(zoneName, trigger);
			return true;
		}
		return false;
	}
	/**
	 * Updates variables in EnemyNPC.
	 * @param zoneName
	 * @param npcId
	 * @param themeTopic
	 */
	public void updateEnemyNPC(String zoneName, int npcId, String themeTopic){
		kingdom.get(zoneName).getEnemyNPC(npcId+"").setTopicTheme(themeTopic);
	}
	/**
	 * Updates variables in TravelTrigger.
	 * @param triggerId
	 * @param name
	 * @param width
	 * @param height
	 * @param destinationTriggerId
	 */
	public void updateTravelTrigger(int triggerId, String name, int width,
			int height, int destinationTriggerId){
		TravelTrigger trig = kingdom.get(getZoneNameForTrigger(triggerId)).getTravelTrigger(triggerId+"");
		trig.setName(name);
		trig.setImageHeight(height);
		trig.setImageWidth(width);
		trig.setDestinationId(destinationTriggerId);
		trig.setDestinationZoneName(getZoneNameForTrigger(destinationTriggerId));
	}
	/**
	 * Helper method for UpdateTravelTrigger.
	 * Returns the name of the zone, trigger with TriggerId is in.
	 * @param triggerId
	 * @return
	 */
	private String getZoneNameForTrigger(int triggerId){
		String zoneName="";
		
		for(Zone z : kingdom.values()){
			if(z.getTravelTrigger(triggerId+"")!=null){
				zoneName = z.getZoneName();
			}
		}
		return zoneName;
	}
	/**
	 * Removes EnemyNPC from zone.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeEnemyNPCFromZone(String zoneName,int npcId){
		kingdom.get(zoneName).removeEnemyNPC(npcId+"");
		worldListener.removeEnemyNPC(zoneName, npcId);
	}
	/**
	 * Removes FriendlyNPC from zone.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeFriendlyNPCFromZone(String zoneName,int npcId){
		kingdom.get(zoneName).removeFriendlyNPC(npcId+"");
		worldListener.removeFriendlyNPC(zoneName, npcId);
	}
	/**
	 * Removes StaticObject from zone.
	 * @param zoneName
	 * @param objId
	 */
	public void removeStaticObjectFromZone(String zoneName,int objId){
		kingdom.get(zoneName).removeStaticObject(objId+"");
		worldListener.removeStaticObject(zoneName, objId);
	}
	/**
	 * Removes TravelTrigger from zone.
	 * @param zoneName
	 * @param triggerId
	 */
	public void removeTravelTriggerFromZone(String zoneName, int triggerId){
		//Check that no triggers are referencing to this trigger
		removeDestinationsTo(triggerId);
		kingdom.get(zoneName).removeTravelTrigger(triggerId+"");
		worldListener.removeTravelTrigger(zoneName, triggerId);
	}
	/**
	 * Removes zone from kingdom.
	 * @param zoneName
	 */
	public void removeZoneFromWorld(String zoneName){
		kingdom.remove(zoneName);
		worldListener.removeZone(zoneName);
	}
	/**
	 * Saves kingdom in database with zoneName as starting zone for new characters.
	 * @param zoneName
	 */
	public void saveKingdomInDatabase(String zoneName){
		StorageHandler.getInstance().saveKingdom(kingdomName, kingdomSubject, kingdomId, kingdom,zoneName,playerAccessList);
	}
	/**
	 * Saves kingdom to harddrive.
	 * @return
	 */
	public boolean saveKingdomToHD(){
		return StorageHandler.getInstance().saveWorldToHarddrive(kingdomName, kingdomSubject, kingdomId, kingdom, null, playerAccessList);
	}
	/**
	 * Saves zone as template.
	 * @param zoneTemplateName
	 * @param zoneName
	 */
	public void saveZoneAsTemplate(String zoneTemplateName){
		if(kingdom.get(selectedZone)!=null){
			StorageHandler.getInstance().saveTemplate(zoneTemplateName,kingdom.get(selectedZone));
		}
	}
	/**
	 * Returns all traveltriggers in kingdom.
	 * @return
	 */
	public ArrayList<TravelTrigger> getAllTravelTriggersInKingdom(){
		ArrayList<TravelTrigger> allTriggers = new ArrayList<TravelTrigger>();
		Iterator<Zone> it = kingdom.values().iterator();
		while(it.hasNext()){
			Zone z = it.next();
			for(TravelTrigger trigger : z.getTravelTriggerList()){
				allTriggers.add(trigger);
			}
		}
		return allTriggers;
	}
	/**
	 * Helpermethod for checking if TravelTrigger with name already exists in kingdom.
	 * @param triggerName
	 * @return
	 */
	private boolean triggerExists(String triggerName){
		Iterator<Zone> it = kingdom.values().iterator();
		while(it.hasNext()){
			Zone z = it.next();
			for(TravelTrigger trigger : z.getTravelTriggerList()){
				if(trigger.getName().equals(triggerName)){
					return true;
				}
			}
		}
		return false;
	}
	/**
	 * Returns name of all zones in kingdom.
	 * @return
	 */
	public String[] getZoneNames(){
		String[] zoneNames = new String[kingdom.size()];
		int i=0;
		for(Zone z : kingdom.values()){
			zoneNames[i] = z.getZoneName();
			i++;
		}
		return zoneNames;
	}
	/**
	 * Helper method for removing traveltriggers.
	 * Checks all traveltriggers and removes destination if deleted travel trigger was their destination.
	 * @param triggerId
	 */
	private void removeDestinationsTo(int triggerId){
		Iterator<Zone> it = kingdom.values().iterator();
		while(it.hasNext()){
			Zone z = it.next();
			for(TravelTrigger trigger : z.getTravelTriggerList()){
				int destination = trigger.getDestinationId();
				if(destination!=0){
					if(destination==triggerId){
						trigger.setDestinationId(0);
						trigger.setDestinationZoneName(null);
					}
				}
			}
		}
	}
	
}
