package lobbyServer.clientSessionHandler;


import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;


import lobbyServer.authenticate.AuthenticationResult;
import lobbyServer.authenticate.AuthenticationResultType;
import lobbyServer.authenticate.ClientAuthenticator;


import shared.models.PlayableCharacter;
import shared.network.ClientSocket;
import shared.network.MessageHandler;
import shared.network.messages.*;
import shared.serverHelpers.*;
/** Main Handler for clients connected to the lobbyserver.
 * 
 * @author Henrik
 *
 */
public class ClientHandler implements MessageHandler{

	

	private ClientFile clientFile;
	private ClientSocket clientSocket;
	private ClientState clientState;
	private ClientAuthenticator authenticator;
	private ArrayList<ShortCharacter> characterList;
	private HashMap<String, Boolean> kingdomAccessList;
	
	
	public ClientHandler(ClientSocket clientSocket, String clientId){
		this.clientSocket = clientSocket;
		this.clientSocket.addMessageListener(this);
		
		clientFile = new ClientFile();
		clientFile.setSessionId(clientId);
		clientState = ClientState.CONNECTED;
	}
	
	public void sendMessage(Message msg){
		clientSocket.sendMessage(msg);
	}
	
	@Override
	public void getMessage(Message message) {
		
		//received login message:
		if(message.getMessageType() == MessageType.LOGIN){
			
			LoginMessage loginmessage = (LoginMessage)message;
			
			handleLogin(loginmessage);
			
		}else if(message.getMessageType() == MessageType.NEW_PLAYABLE_CHARACTER){
			NewCharacterFromClientMessage charMsg = (NewCharacterFromClientMessage)message;
			//Creates playableCharacter
			PlayableCharacter character = new PlayableCharacter(charMsg.getCharacterName(),charMsg.getSex(),charMsg.getCharacterClass());
						
			LobbyControl.getDbConnection().sendMessage(new NewPlayableCharacterMessage("Lobby","DBServer","Insert character to db",character,clientFile.getUserName(),charMsg.getKingdomName(),clientFile.getSessionId()));
		}
		//received DB player response
		else if(message.getMessageType() == MessageType.OBJECT_MESSAGE){
			
			ObjectMessage omsg = (ObjectMessage)message;
			if(omsg.getObjectType() == ObjectType.PLAYER || omsg.getObjectType() == ObjectType.IDENTIFICATION_ERROR){
				authenticator.dbResponseReceived(omsg);
			}
			else if(omsg.getObjectType() == ObjectType.SHORT_CHARACTER_LIST){

				characterList = (ArrayList<ShortCharacter>)omsg.getObject();
				
				if(kingdomAccessList != null){

					sendWorldAndCharacterInfo();
				}
			}
			else if(omsg.getObjectType() == ObjectType.KINGDOM_ACCESS_LIST){
				
				kingdomAccessList = (HashMap<String,Boolean>) omsg.getObject();
				
				if(characterList != null){

					sendWorldAndCharacterInfo();
				}
			}
		}
	}
	
	
	public void lostConnection(){
		if(clientState != ClientState.DISCONNECTING && clientState != ClientState.DISCONNECTED){
			System.out.println("Lost client: " + clientFile.getSessionId());
			LobbyControl.getLobbyControl().removeClientHandler(clientFile.getSessionId());
		}
	}
	
	public boolean disconnectClient(String message){
		
		this.clientState = ClientState.DISCONNECTING;
		clientSocket.sendMessage(new Message("LOBBYSERVER", clientFile.getSessionId(), message, MessageType.CLOSE_CONNECTION));
		clientSocket.disconnect();
		
		LobbyControl.getLobbyControl().removeClientHandler(clientFile.getSessionId());
		
		this.clientState = ClientState.DISCONNECTED;
		return true;
	}
	
	private void handleLogin(LoginMessage loginmessage){
		
		
		
		if(authenticator == null){
			authenticator = new ClientAuthenticator();
			
		}
		AuthenticationResult res = authenticator.Authenticate(loginmessage.getUserName(), loginmessage.getPassword(), clientFile.getSessionId());
		
		boolean authOk;
		
		if(res.getType() == AuthenticationResultType.OK){
			
			clientFile.setUserName(loginmessage.getUserName());
			authOk = true;
			clientFile.setPlayerObject(res.getPlayer());
			clientState = ClientState.LOGGED_IN;
			
			requestShortCharacterList();
			
			requestKingdomAccessList();
			
		}
		else{
			
			authOk = false;
			
		}
		LoginResult resMessage = new LoginResult("LOBBYSERVER", clientFile.getSessionId(), res.getAuthenticationMessage(), authOk);
		
		
		clientSocket.sendMessage(resMessage);
		
	}
	
	public void printOverview(){
		System.out.println(clientFile.getSessionId() + ": " + clientFile.getUserName() + " " + clientState);
	}
	
	// This method is used by the LobbyControl to check for multiple entities of the same player.
	// It returns true if the playername matches the object's playername, AND the user is
	// logged in!
	public boolean checkPlayernameAndConnectionstatus(String playerName){
		if(clientFile.getUserName().equals(playerName)){
			if(clientState == ClientState.LOGGED_IN || clientState == ClientState.GOT_SERVERLIST || clientState == ClientState.JOINED_SERVER){
				return true;
			}
		}
		return false;
	}
	
	public void banPlayer(String reason){
		BanInfo bInfo = new BanInfo(clientFile.getPlayerObject().getPlayerId(), reason, new Timestamp(System.currentTimeMillis()), 0, clientFile.getUserName());
		LobbyControl.getLobbyControl().addBan(bInfo);
	}
	
	public void requestShortCharacterList(){
		LobbyControl.getDbConnection().sendMessage(new RequestShortCharacterListMessage("LOBBYSERVER", "DBSERVER", "REQUEST SHORTCHARACTERLIST", clientFile.getUserName(), clientFile.getSessionId()));
	}
	
	public void requestKingdomAccessList(){
		LobbyControl.getDbConnection().sendMessage(new RequestKingdomAccessMessage("LOBBYSERVER", "DBSERVER", "REQUEST KINGDOMACCESS", clientFile.getSessionId(), clientFile.getUserName()));
	}
	
	public void sendWorldAndCharacterInfo(){
		//TODO: get and check "AccessToKingdom" from db
		ArrayList<ClientKingdomInfo> kdominfoList = new ArrayList<ClientKingdomInfo>();
		
		ArrayList<WorldServer> serverList = LobbyControl.getLobbyControl().getWorldServers();
		
		for (WorldServer worldServer : serverList) {
			
			if(kingdomAccessList.get(worldServer.getKingdomName()) != null){
				ClientKingdomInfo kdominfo = new ClientKingdomInfo(worldServer, "online");
				String kingdomName = worldServer.getKingdomName();
				
				for (ShortCharacter character : characterList) {
					if(character.getKingdomKey().equals(kingdomName)){
						kdominfo.addCharacter(character);
					}
				}
				
				kdominfoList.add(kdominfo);
			}

		}
		
		clientSocket.sendMessage(new ObjectMessage("LOBBYSERVER", clientFile.getSessionId(), "SERVER AND CHARACTER LIST", MessageType.OBJECT_MESSAGE, kdominfoList, ObjectType.KINGDOM_INFO_LIST, clientFile.getSessionId()));
		
	}
	
	public boolean loginIsCorrectAndAllIsWell(String worldserver, String characterName){
		//TODO: Check against "AccessToKingdom" (from db)
		
		//checking if the client has been logged in:
		if(clientState == ClientState.LOGGED_IN || clientState == ClientState.GOT_SERVERLIST){
			
			//check if the charactername belongs to this player on that server:
			for (ShortCharacter character : characterList) {
				if(character.getKingdomKey().equals(worldserver) && character.getName().equals(characterName)){
					return true;
				}
			}
			
		}
		
		return false;
	}
	
	
	
}
