package lobbyServer.clientSessionHandler;
import lobbyServer.network.*;
import shared.fileHandler.Settings;
import shared.models.Player;
import shared.network.ClientSocket;
import shared.network.messages.*;
import shared.serverHelpers.*;

import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.sun.jmx.remote.internal.ClientListenerInfo;

/**
 * 
 * Starts the network listener, and creates a connection to DB.
 */
public class LobbyControl{

	private HashMap<String, ClientHandler> clientList;
	private HashMap<String, WorldServerHandler> worldServerList;
	private int lastClientNumber;
	private int lastServerNumber;
	private HashMap<String, BanInfo> banList;
	
	
	
	private static DatabaseConnection dbConnection;
	private static LobbyControl controlSingleton;
	private static Settings networkSettings;
	public static LobbyControl getLobbyControl(){
		
		if(controlSingleton == null){
			controlSingleton = new LobbyControl();
		}
		
		return controlSingleton;
	}
	
	private LobbyControl(){
		lastClientNumber = 0;
		lastServerNumber = 0;
		networkSettings = new Settings("lobbyServer");
		clientList = new HashMap<String, ClientHandler>();
		worldServerList = new HashMap<String, WorldServerHandler>();
		
		dbConnection = new DatabaseConnection( networkSettings.getDatabaseIp(), networkSettings.getDatabasePort()); //Connect to Database server
		
		dbConnection.sendMessage(new Message("LOBBYSERVER", "DBSERVER", "Request Banlist", MessageType.GET_BAN_LIST));
		
		new ClientListener(networkSettings.getClientListenPort());
		new WorldServerListener(networkSettings.getWorldServerListenPort());
	}
	/**
	 * Broadcast messages to the clients.
	 * @param msg
	 */
	public void broadcastMessage(Message msg){

		Iterator<ClientHandler> it = clientList.values().iterator();
		while(it.hasNext()){
			it.next().sendMessage(msg);
		}
	}
	/**
	 * Called when a new client has connected.
	 * @param newsocket
	 */
	public void newClientConnectionReceived(Socket newsocket){
		lastClientNumber++;
		String clientID = "client"+lastClientNumber;
		clientList.put(clientID, new ClientHandler(new ClientSocket(newsocket),clientID));
		
		System.out.println("New client added");
	}
	/**
	 * Called when a new world server has connected.
	 * @param newsocket
	 */
	public void newWorldServerConnectionReceived(Socket newsocket){
		
		lastServerNumber++;
		String serverID = "worldserver" + lastServerNumber;
		WorldServerHandler wsHandler = new WorldServerHandler(new ClientSocket(newsocket), serverID);
		worldServerList.put(serverID, wsHandler);

		System.out.println("New worldserver added");
	}
	
	public int getNumberOfConnectedPlayers(){
		return clientList.size();
	}
	
	public static DatabaseConnection getDbConnection(){
		
		if(dbConnection == null){
			
			dbConnection = new DatabaseConnection( networkSettings.getDatabaseIp(), networkSettings.getDatabasePort());//Connect to Database server
		}
		
		return dbConnection;
	
	}
	
	public void receiveDBMessage(Message msg){
		
		if(msg.getMessageType() == MessageType.OBJECT_MESSAGE){
			ObjectMessage omsg = (ObjectMessage) msg;
			if(omsg.getObjectType() == ObjectType.PLAYER || omsg.getObjectType() == ObjectType.IDENTIFICATION_ERROR){
				
					clientList.get(omsg.getSessionId()).getMessage(omsg);
				
			}
			else if(omsg.getObjectType() == ObjectType.SHORT_CHARACTER_LIST){
				
					clientList.get(omsg.getSessionId()).getMessage(omsg);
					
			}
			else if(omsg.getObjectType() == ObjectType.BANNED_PLAYER_LIST_HASH){
				//System.out.println("Got Banlist");
				banList = (HashMap<String, BanInfo>)omsg.getObject();
			}
			else if(omsg.getObjectType() == ObjectType.KINGDOM_ACCESS_LIST){
				
				clientList.get(omsg.getSessionId()).getMessage(omsg);
			}
		
		}

		
		
	}
	
	public void printClientList(){
		
		System.out.println("------Clients:------");
		
		Iterator<ClientHandler> it = clientList.values().iterator();
		while(it.hasNext()){
			it.next().printOverview();
		}
		
		System.out.println("--------------------");
		
	}
	
	public void printServerList(){
		
		System.out.println("------Servers:------");
		
		Iterator<WorldServerHandler> it = worldServerList.values().iterator();
		while(it.hasNext()){
			it.next().printOverview();
		}
		
		System.out.println("--------------------");
	}
	
	public void disconnectClients(String message){
		
		System.out.println("------Disconnecting Clients-----");
		
	}
	
	public void removeClientHandler(String sessionID){
		System.out.println("removing clienthandler");
		clientList.remove(sessionID);
	}
	
	public void removeServerHandler(String sessionID){
		System.out.println("removing serverhandler");
		worldServerList.remove(sessionID);
	}
	
	public synchronized boolean clientAlreadyConnected(String userName){
		
		Iterator<ClientHandler> it = clientList.values().iterator();
		while(it.hasNext()){
			if(it.next().checkPlayernameAndConnectionstatus(userName)){
				return true;
			}
		}
		
		return false;
	}
	
	/*
	 * Method to check if a server registration is ok, returns true if the server name or
	 * ipadress/port combination is already present in the server list!
	 */
	public synchronized boolean serverAlreadyRegistered(String serverName, String ip, int port){
		
		Iterator<WorldServerHandler> it = worldServerList.values().iterator();
		while(it.hasNext()){
			if(it.next().checkAgainstWorldServerConnection(serverName, ip, port)){
				return true;
			}
		}
		
		return false;
		
	}
	
	public ClientHandler getClientHandler(String sessionId){
		
		return clientList.get(sessionId);
		
	}
	/**
	 * Checks the ban list if the player has been banned.
	 * @param playerName
	 * @return
	 */
	public BanInfo checkBanList(String playerName){
		
		return banList.get(playerName);
		
	}
	/**
	 * Adds a ban to the Database.
	 * @param banInfo
	 */
	public void addBan(BanInfo banInfo){
		banList.put(banInfo.getPlayerId()+"", banInfo);
		getDbConnection().sendMessage(new ObjectMessage("LOBBYSERVER", "DBSERVER", "Add new ban", MessageType.BAN_PLAYER, banInfo, ObjectType.BANNED_INFO, null));
	}
	
	public void banPlayer(String sessionId, String reason){
		clientList.get(sessionId).banPlayer(reason);
		
	}
	
	public void unBanPlayer(String playerName){
		
				getDbConnection().sendMessage(new ObjectMessage("LOBBYSERVER", "DBSERVER", "Remove ban", MessageType.UNBAN_PLAYER, banList.get(playerName), ObjectType.BANNED_INFO, null));
				banList.remove(playerName);
				
	}
	
	public void printBanList(){
		
		System.out.println("------Banlist:------");
		Iterator<BanInfo> it = banList.values().iterator();
		while(it.hasNext()){
			BanInfo b = it.next(); 
			System.out.println(b.getBanId() + ":" + b.getPlayerName() + "  - " + b.getReason());
			
		}
		
		System.out.println("--------------------");
	}
	
	public ArrayList<WorldServer> getWorldServers(){
		
		ArrayList<WorldServer> list = new ArrayList<WorldServer>();
		
		Iterator<WorldServerHandler> it = worldServerList.values().iterator();
		while(it.hasNext()){

			WorldServer ws = it.next().getWorldServerIfServerRegistered();
			if(ws != null){
				list.add(ws);
			}
			
		}
		
		
		return list;
		
	}
	
	public boolean checkPlayer(String sessionId, String playerName, String kingdomName, String characterName){
		
		if(clientList.containsKey(sessionId)){
			return clientList.get(sessionId).loginIsCorrectAndAllIsWell(kingdomName, characterName);
				
		}
		
		return false;
		
	}
	
}
