package worldServer.gameLoop;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;


import shared.models.AttackType;
import shared.models.BackgroundTile;
import shared.models.EnemyNPC;
import shared.models.EventType;
import shared.models.FriendlyNPC;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.SimpleCharacter;
import shared.models.SimpleZone;
import shared.models.StaticObject;
import shared.models.Character;
import shared.models.TravelTrigger;
import shared.models.Zone;
import shared.network.messages.actions.*;
/**
 * GameLoops keeps an overview of one zone, and handles players
 * and creatures within the zone. 
 * @author thorgrun
 */
public class GameLoop implements GameLoopInterface{
	private UpdateController gameTimer;
	private HashMap<Integer, ArrayList<Event>> eventList; //Contains a list of actions to be done at different times.
	

	private Zone zone;
	private int currentTime;
	private GameWorld game;
	
	/**
	 * Creates a GameLoop
	 * @param zoneId id of the Zone
	 * @param backgroundArray Array of background tiles
	 * @param respawnPoint The location where players may respawn in the zone
	 * @param game reference to the GameWorld object
	 */
	public GameLoop(Zone zone,GameWorld game){
		this.zone = zone;
		//TODO: Get objects and locations, respawnpoint, zone transitions
		this.game = game;
		gameTimer = new UpdateController(this);
		this.eventList = new HashMap<Integer, ArrayList<Event>>();
		
		
	}
	/**
	 * Puts a client in the zone
	 * @param clientId
	 * @param character
	 */
	public void putClientInZone(String clientId,PlayableCharacter character){
		JoinZoneMessage joinMsg = new JoinZoneMessage("WorldServer",clientId,character.getName()+" has joined.",character.getName(),zone.getZoneId(),character.getSimpleCharacter());
		sendActionUpdateToClients(joinMsg);
		zone.putClientInZone(clientId, character);
		
	}
	public Location getTravelTriggerPosition(int triggerId){
		return zone.getTravelTrigger(triggerId+"").getCurrentLocation();
	}
	public void putClientAtTriggerInZone(String clientId,PlayableCharacter character,int triggerId){
		JoinZoneMessage joinMsg = new JoinZoneMessage("WorldServer",clientId,character.getName()+" has joined.",character.getName(),zone.getZoneId(),character.getSimpleCharacter());
		sendActionUpdateToClients(joinMsg);
		character.setCurrentLocation(zone.getTravelTrigger(triggerId+"").getCurrentLocation());
		zone.putClientInZone(clientId, character);
	}

	/**
	 * Handles action messages that comes from players.
	 * @param clientId
	 * @param actionMsg
	 */
	public void action(String clientId,ActionMessage actionMsg){
		switch(actionMsg.getActionType()){
		case MOVEMENT:
			MovementMessage movMsg = (MovementMessage)actionMsg;
			//TODO: Check if path is cleared of obstacles
			//Assumes okay and send messages to all
			zone.setClientCurrentAndTargetLocation(movMsg.getSourceId(), movMsg.getCurrentLocation(), movMsg.getTargetLocation());
			sendActionUpdateToClients(movMsg);
			break;
		case ATTACK:
			AttackMessage attMsg = (AttackMessage)actionMsg;
			if(attMsg.getBaseDamage() == 0){
				//missed
				attMsg.setSuccessfullAttack(false);
				sendActionUpdateToClients(new UpdateHealthMessage("WORLDSERVER","","Target was not hit.",attMsg.getAttackedId(),false,0));
			}
			else{
				calculateAttack(attMsg,clientId);
				
				if(attMsg.isDodgedAttack()){
					sendActionUpdateToClients(new UpdateHealthMessage("WORLDSERVER","","Target was not hit.",attMsg.getAttackedId(),false,0));
				}else{
					updateHealth(zone.getClientCharacter(clientId),attMsg.getAttackedId(), attMsg.getDamage());
					
				}
				System.out.println("!!!!!!!!!!!! total : " + attMsg.getDamage());
			}
			break;
			
		case JOIN_ZONE:
			
			break;
		case CHANGE_ZONE:
			
			break;
		}
	}
	/**
	 * Deals damage to a Character. 
	 * Returns true if target dies.
	 * @param attacker
	 * @param attackedId
	 * @param damage
	 */
	private boolean updateHealth(Character attacker,String attackedId,int damage){
		
		Character target = zone.getEnemyNPC(attackedId);
		if(target==null){
			target = zone.getClientCharacter(attackedId);
		}
		if(target instanceof PlayableCharacter){
			attackedId = target.getName();
		}
		if(target.updateHP(damage)){//Updates EnemyHP and returns true if dead.
			if(target instanceof EnemyNPC){
				addEvent(60,target , EventType.RESPAWN);
			}
			sendActionUpdateToClients(new IsDeadMessage("WORLDSERVER","","Target was killed by "+attacker.getName(),attackedId,attacker.getName()));
			return true;
		}else{
			sendActionUpdateToClients(new UpdateHealthMessage("WORLDSERVER","","Target was hit for "+damage,attackedId,true,target.getAttribute().getHP()));
			return false;
		}
	}
	private void calculateAttack(AttackMessage attMsg,String clientId){
		//calculate damage
		attMsg.setSuccessfullAttack(true);
		//check if the target dodges
		if(zone.getEnemyNPC(attMsg.getAttackedId()).
				dodgeSuccessful(attMsg.getQuestionLevel())){
			attMsg.setDodgedAttack(true);
		}
		else{
			attMsg.setDodgedAttack(false);
			//calculate damage given
			attMsg.updateDamage(zone.getClientCharacter(clientId).
					calculateDamgeDealt(
							attMsg.getAttributeType(),
							attMsg.getAttackType(),
							attMsg.getEffect(),
							attMsg.getDamageType()));
			if(attMsg.getAttackType() == AttackType.PHYSICAL){
				attMsg.setDamageType(zone.getClientCharacter(clientId).getDamageTypeOnLastAttack());
			}
			System.out.println(attMsg.getDamageType());
			System.out.println("!!!!!!!!!!!!!!!!!!!!! attack : " + attMsg.getDamage());
			//calculate defence
			int defence = zone.getEnemyNPC(attMsg.getAttackedId()).
								calculateDefence(attMsg.getDamageType());
			if(defence < attMsg.getDamage()){
				attMsg.updateDamage(-defence);
			}
			else if(defence == attMsg.getDamage()){
				//the attack does 0 damage
				attMsg.updateDamage(-defence);
				//the attack will do at least 1 damage
				attMsg.updateDamage(1);
			}
			else if(defence > attMsg.getDamage()){
				attMsg.updateDamage(-attMsg.getDamage());
				//the attack will do at least 1 damage
				attMsg.updateDamage(1);
			}
			System.out.println("!!!!!!!!!!!!!!!!!!! defence: " + defence);
		}
	}
	
	private void sendActionUpdateToClients(ActionMessage actionMsg){
		System.out.println("Broadcasts action to players");
		Iterator<String> it = zone.getClientListIterator();
		while(it.hasNext()){
			game.sendMessageToClient(it.next(), actionMsg);
		}
	}
	/**
	 * Get client in zone
	 * @param clientId
	 * @return Client
	 */
	public PlayableCharacter getClient(String clientId){
		return	zone.getClientCharacter(clientId);
	}
	
	/**
	 * Returns a character in list and removes it from the list
	 */
	public PlayableCharacter retrieveClient(String clientId){
		PlayableCharacter temp=zone.getClientCharacter(clientId);
		removeClient(clientId);
		return temp;
	}
	/**
	 * Removes client from zone
	 * @param clientId
	 */
	public void removeClient(String clientId){
		PlayableCharacter temp=zone.getClientCharacter(clientId);
		
		zone.getSimpleZone().getCharacterList().remove(temp.getName());
		zone.removeCharacter(clientId);
		LeaveZoneMessage leaveMsg = new LeaveZoneMessage("WorldServer",clientId,temp.getName()+" left the game.",temp.getName());
		sendActionUpdateToClients(leaveMsg);
	}
	
	/**
	 * Add an event to the eventList.
	 * @param inXseconds how many seconds to the event will run
	 * @param object Object that is used in event
	 * @param type Type of event
	 */
	public void addEvent(int inXseconds, Character object, EventType type){
		//TODO: perhaps not take in seconds as param, but find by what event type it is.
		if(eventList.get(currentTime+inXseconds)==null){
			ArrayList<Event> temp = new ArrayList<Event>();
			temp.add(new Event(object,type));
			eventList.put(currentTime+inXseconds,temp);
		}else{
			eventList.get(currentTime+inXseconds).add(new Event(object,type));
		}
	}
	

	/**
	 * Main gameloop. Is continuously run.
	 * Does an update check one time per second.
	 */
	public void update(long elapsedTime) {
		//Check if monster should do damage to a player,
		//Check if timers of spells/effekts has run out..
		//Check if dead monster should respawn
		//Check if things in general is syncronized, syncronize every X seconds?
		if(currentTime!= gameTimer.getTotalElapsedTimeInSec()){
			currentTime = gameTimer.getTotalElapsedTimeInSec();
			zone.updatePositions(1000);
			//runs 1 per second
			if(eventList.get(currentTime)!=null){
				for(Event event: eventList.get(currentTime)){
					if(event.getTypeOfEvent()==EventType.ATTACK){
						//Do damage to target player
						//TODO: Check if player is within range.
						if(event.getEventTarget() instanceof EnemyNPC){
							EnemyNPC npc = (EnemyNPC)event.getEventTarget();
							//Check if player is still in the zone.
							if(npc.getTargetId()!=null&&zone.getClientCharacter(npc.getTargetId())!=null){
								if(npc.isAlive()){
									int damage = npc.calculateDamage();
									if(!updateHealth(npc, npc.getTargetId(), damage)){
										//if the player does not die, schedule new attack
										addEvent(npc.getSleepTime(),npc,EventType.ATTACK);
									}else{
										npc.setTargetId(null);
										//TODO:If player dies, give respawn options.
									}
								}else{
									npc.setTargetId(null);
								}
							}else{
								npc.setTargetId(null);
							}
							
						}
					}else{
						event.runEvent();
					}
				}
				eventList.remove(currentTime);
			}
		}
	}
	
	public void addStaticObject(String id,StaticObject object){
		zone.addStaticObject(id, object);
	}
	
	public void addFriendlyNPC(String id,FriendlyNPC npc){
		zone.addFriendlyNPC(id, npc);
	}
	
	public void addEnemyNPC(String id,EnemyNPC npc){
		zone.addEnemyNPC(id, npc);
	}
	public void addTrigger(String id,TravelTrigger trigger){
		zone.addTravelTrigger(id, trigger);
	}
	public EnemyNPC getEnemyNPC(String enemyId){
		return zone.getEnemyNPC(enemyId);
	}
	public FriendlyNPC getFriendlyNPC(String friendlyId){
		return zone.getFriendlyNPC(friendlyId);
	}
	public PlayableCharacter getClientCharacter(String clientId){
		return zone.getClientCharacter(clientId);
	}
	public String getZoneId(){
		return zone.getZoneId();
	}
	public SimpleZone getSimpleZone(){
		return zone.getSimpleZone();
	}
	
	
	
}