package worldServer.gameLoop;
import java.util.ArrayList;
import java.util.HashMap;
import shared.fileHandler.ReadBackgroundFromFile;
import shared.models.Answer;
import shared.models.Attributes;
import shared.models.EnemyNPC;
import shared.models.EventType;
import shared.models.FriendlyNPC;
import shared.models.Gender;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.Question;
import shared.models.SimpleZone;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
import shared.network.messages.Message;
import shared.network.messages.QuestionMessage;
import shared.network.messages.RequestQuestionMessage;
import shared.network.messages.actions.ActionMessage;
import shared.network.messages.actions.ActionType;
import shared.network.messages.actions.AttackMessage;
import shared.network.messages.actions.ChangeZoneMessage;
import shared.network.messages.actions.SendAnswerMessage;
import worldServer.messageHandler.ClientHandler;
import worldServer.messageHandler.MessageHandlerForWorld;
import worldServer.serverControls.ChatSystem;


/**
 * This class contains the GameWorld, which is a list of zones,
 * and a list of the players that has connected to the world and has been authorized.
 * 
 * @author thorgrun
 *
 */
public class GameWorld implements MessageHandlerForWorld{
	
	private HashMap<String,GameLoop> zoneList;
	private HashMap<String,ClientHandler> clientList;
	private GameWorldState state;
	private ArrayList<Question> questionList;//List of all possible questions
	//will track the different clients question progression
	private HashMap<String, QuestionAnswerChecker> questionProgression;

	private ChatSystem chat;
	/**
	 * Creates a gameworld.
	 * 
	 */
	public GameWorld(HashMap<String, Zone> kingdom){
		this.chat = new ChatSystem(this);
		this.clientList = new HashMap<String,ClientHandler>();
		this.zoneList = new HashMap<String, GameLoop>();
		this.state = GameWorldState.EMPTY;
		this.questionProgression = new HashMap<String, QuestionAnswerChecker>();
	
		for(Zone z : kingdom.values()){
			addZone(z.getZoneName(),new GameLoop(z,this));
		}
		state = GameWorldState.READY;
	}
	


	public ArrayList<Question> getQuestionList(){
		return questionList;
	}

	
	public void setQuestionList(ArrayList<Question> questionList) {
		this.questionList = questionList;
		System.out.println("Got questionList");
	}
	
	public GameWorldState getState() {
		return state;
	}
	
	public void setState(GameWorldState state) {
		this.state = state;
	}
	public void addZone(String zoneId,GameLoop zone){
		System.out.println(zoneId);
		this.zoneList.put(zoneId, zone);
	}
	public SimpleZone getSimpleZone(String zoneId){
		return zoneList.get(zoneId).getSimpleZone();
	}
	/**
	 * Adds a client to the GameWorld. Added after the client has been authorized.
	 * @param clientId
	 * @param client
	 */
	public void addClientToWorld(String clientId, ClientHandler client){
		this.clientList.put(clientId, client);
		chat.addClientToChannel(clientId, "Main");
		client.addMessageListener(this);
	}
	
	/**
	 * Sends a message to client.
	 */
	public void sendMessageToClient(String clientId,Message msg){
		msg.setTargetId(clientId);
		clientList.get(clientId).sendMessage(msg);
	}
	
	public void addClientToZone(String clientId,String zoneId,PlayableCharacter cha){
		zoneList.get(zoneId).putClientInZone(clientId, cha);
	}
	
	public void moveClientToNewZone(String clientId, String currentZoneId, String newZoneId,int triggerId){
		zoneList.get(newZoneId).putClientAtTriggerInZone(clientId,zoneList.get(currentZoneId).retrieveClient(clientId),triggerId);
	}
	
	private void clientAction(String clientId,String zoneId,ActionMessage actionMsg){
		zoneList.get(zoneId).action(clientId,actionMsg);
	}
	/**
	 * Used for status in servercontrol,TODO:remove later
	 */
	public HashMap<String, ClientHandler> getPlayerList(){
		return clientList;
	}
	
	/**
	 * Receives messages from clients.
	 */
	public void getMessage(String zoneId,Message message) {
		switch(message.getMessageType()){
		case CHATMESSAGE:
			chat.receivedMessage(message);
			break;
		case ACTION_MESSAGE:
			ActionMessage actionMsg = (ActionMessage)message;
			if(actionMsg.getActionType()==ActionType.CHANGE_ZONE){
				ChangeZoneMessage changeMsg = (ChangeZoneMessage)actionMsg;
				clientList.get(message.getSourceId()).movedToZone(changeMsg.getDestinationZone());
				moveClientToNewZone(message.getSourceId(),zoneId , changeMsg.getDestinationZone(), changeMsg.getDestinationId());
				//Update player position
				clientList.get(message.getSourceId()).newPosition(zoneList.get(changeMsg.getDestinationZone()).getTravelTriggerPosition(changeMsg.getDestinationId()));					
			}else{
				clientAction(message.getSourceId(),zoneId,actionMsg);
			}
			break;
		case REQUEST_QUESTION:
			RequestQuestionMessage reqQuestionMsg = (RequestQuestionMessage)message;
			Question selectedQuestion = getARelevantQuestion(reqQuestionMsg.getAttack().getQuestionLevel(),
															zoneList.get(zoneId).getEnemyNPC(reqQuestionMsg.getAttackTargetId()).getTopicTheme());
					
			QuestionMessage questionmessage = createQuestionMessage(
												//we switches the target and source because this
												//will be sent in the opposite direction
												reqQuestionMsg.getTargetId(),
												reqQuestionMsg.getSourceId(),
												selectedQuestion);
			EnemyNPC attackedNPC = zoneList.get(zoneId).getEnemyNPC(reqQuestionMsg.getAttackTargetId());
			sendMessageToClient(reqQuestionMsg.getSourceId(),questionmessage);
			if(!(questionProgression.containsKey(reqQuestionMsg.getSourceId()))){
				//the client has not gotten any questions from this attack before
				QuestionAnswerChecker checkAnswer = new QuestionAnswerChecker(reqQuestionMsg.getAttack(),
													attackedNPC.getTopicTheme(),
													questionmessage.getTimeLimit(),
													reqQuestionMsg.getAttackTargetId()
													);
				//sets the correct answers
				checkAnswer.setCurrentQuestion(selectedQuestion);
				attackedNPC.setTargetId(reqQuestionMsg.getSourceId());
				//NPC attacks back in 30 seconds
				zoneList.get(zoneId).addEvent(attackedNPC.getSleepTime(), attackedNPC, EventType.ATTACK);
				questionProgression.put(reqQuestionMsg.getSourceId(),checkAnswer);
			}
			break;
			
		case ANSWER_QUESTION:
			SendAnswerMessage answerMessage  = (SendAnswerMessage)message;
			QuestionAnswerChecker qa = questionProgression.get(answerMessage.getSourceId());
			AttackMessage attackMessage;
			if(qa != null){
				qa.checkRightAnswer(answerMessage.getAnswerText(), 
						answerMessage.getTimeLeft());
				if(qa.checkAttackComplete()){
					//the attack is complete
					System.out.println("Attack complete");
					if(qa.checkIfAttackSuccede()){
						System.out.println("Hit");
						attackMessage = new AttackMessage(answerMessage.getTargetId(),
														answerMessage.getSourceId(),
														"Sending damage",
														"",
														qa.calculateDamage(),
														qa.getDamageType(),
														answerMessage.getSourceId(),
														qa.getAttackedTarget(),
														qa.getAttriubute(),
														qa.getAttackType(),
														qa.getEffect(), 
														qa.getQuestionLevel(),
														qa.getManaCost());
						clientAction(message.getSourceId(), zoneId, attackMessage);
					}
					else{
						System.out.println("Missed");
						attackMessage = new AttackMessage(answerMessage.getTargetId(),
								answerMessage.getSourceId(),
								"Sending damage",
								"",
								0,
								qa.getDamageType(),
								answerMessage.getSourceId(),
								qa.getAttackedTarget(),
								qa.getAttriubute(),
								qa.getAttackType(),
								qa.getEffect(),
								qa.getQuestionLevel(),
								qa.getManaCost());
						clientAction(message.getSourceId(), zoneId, attackMessage);
					}
					//we have answered enough questions in this attack, and we delete this from
					//the hashmap
					questionProgression.remove(answerMessage.getSourceId());
				}
				else{
					//send a new question
					Question newQuestion = getARelevantQuestion(qa.getQuestionLevel(),qa.getTopic());
					QuestionMessage anotherQuestionMessage = createQuestionMessage(
							answerMessage.getTargetId(),
							answerMessage.getSourceId(),
							newQuestion
							);
					sendMessageToClient(answerMessage.getSourceId(),anotherQuestionMessage);
					qa.setCurrentQuestion(newQuestion);
				}
			}
			break;
		}	
	}
	
	/**
	 * Is called when a client looses connection.
	 */
	public void lostConnection(String clientId) {
		String zoneId = clientList.get(clientId).getInZone();
		chat.removeClientFromAllChannels(clientId);
		zoneList.get(zoneId).removeClient(clientId);
		clientList.remove(clientId);
	}
	
	/**
	 * gets an arraylist containing the relevant question based on
	 * level and topic.
	 * If there is no match between level and topic, there will be 
	 * returned a list where either level or topic match.
	 * If there is no match on either level or topic, it returns 
	 * a random question from the list
	 * @param level - the level on the question
	 * @param topic - the question topic
	 * @return - an arraylist with relevant (or semi relevant, or non relevant question)
	 */
	private Question getARelevantQuestion(int level, String topic){
		ArrayList<Question> relevantQuestion = new ArrayList<Question>();
		for(Question q: questionList){
			if(q.getQuestionLevel()  == level){
				if(q.getQuestionTopic().equals(topic)){
					relevantQuestion.add(q);
				}
			}
		}
		// if there is no relevant questions, find a semi-relevant question, based on topic
		//or difficulty
		if(relevantQuestion.size() < 1){
			for(Question q : questionList){
				if(q.getQuestionLevel() == level || q.getQuestionTopic().equals(topic)){
					relevantQuestion.add(q);
				}
			}
		}
		//there is no question which match the level or topic,
		//we just take a random question from the list 
		if(relevantQuestion.size() < 1){
			//take a random question from the questionList
			relevantQuestion.add(questionList.get((int)(Math.random()*questionList.size())));
		}
		//takes a random Question from the relevantQuestion list
		Question selectedQuestion = relevantQuestion.get((int) (Math.random()*relevantQuestion.size()));
		
		return selectedQuestion;
	}
	
	/**
	 * This will create a question message
	 * @param sourceId - from
	 * @param targetId - to 
	 * @param level - the questionLevel
	 * @param topic - the question topic
	 * @return - a questionMessage
	 * @see QuestionMessage
	 */
	private synchronized QuestionMessage createQuestionMessage(String sourceId, String targetId,Question selectedQuestion){
		
		//this will contain all the answers
		ArrayList<String> answers = new ArrayList<String>();
		//randomizes the questions
		ArrayList<Answer> tempAnswerList = new ArrayList<Answer>(selectedQuestion.getAnswers().size());
		for(Answer a : selectedQuestion.getAnswers()){
			tempAnswerList.add(a);
		}
		//this index is the posision to the answer in the answerList
		int index =0;
		while(tempAnswerList.size() >0){
			int randomIndex = (int)(Math.random()*tempAnswerList.size());
			answers.add(tempAnswerList.get(randomIndex).getAnswerText());
			tempAnswerList.remove(randomIndex);
			index ++;
		}
		String message ="Here is a message with an question, bound for the questionPanel";
		return new QuestionMessage(sourceId,targetId,message,selectedQuestion.getQuestionID(),
				selectedQuestion.getTimeLimit(),selectedQuestion.getQuestionText(),
				selectedQuestion.getQuestionHeading(),answers);
	}
}