package worldServer.gameLoop;

import shared.models.Answer;
import shared.models.Attack;
import shared.models.AttackType;
import shared.models.AttributeTypes;
import shared.models.DamageType;
import shared.models.Effect;
import shared.models.Question;
import client.graphics.components.QuestionPanel;

/**
 * This class will contain all the information and logic 
 * which will decide if a client answers correct or wrong on 
 * question, and keep track of how many question one attack needs
 * @author Esben Fllesdal
 *
 */
public class QuestionAnswerChecker {


	private int numberOfQuestionsLeftToAnswer;
	private int numberOfCorrectAnswers = 0;
	private int numberOfCorrectAnswersNeededToScuccessfullyMakeAnAttack;
	private String topic;
	private int usedTime = 0; 
	private Question currentQuestion;
	private int availableTimeLastQuestion = 0;
	private int totalAvailableTime = 0;
	private String attackedTarget; // the id to the character which is targeted with the attack
	private Attack attackSkill;
	public QuestionAnswerChecker(Attack attack, 
			String topic,  int availableTime, String attackedTarget){
		this.attackSkill = attack;
		 this.numberOfCorrectAnswersNeededToScuccessfullyMakeAnAttack =
			//numberOfCorrectAnswersNeededToScuccessfullyMakeAnAttack is given as percent, have 
		    //to convert it to an int
	    (int)(((attackSkill.getAnswerPercentage()/100)*attackSkill.getNumberOfQuestions())+0.5);
		 //sets the number of questions which must be answered
		 this.numberOfQuestionsLeftToAnswer = attackSkill.getNumberOfQuestions();
		 this.topic = topic;
		 this.availableTimeLastQuestion = availableTime;
		 this.attackedTarget = attackedTarget;
	}
	
	public AttackType getAttackType() {
		return attackSkill.getAttackType();
	}

	public int getManaCost(){
		return attackSkill.getMana();
	}
	public Effect getEffect() {
		return attackSkill.getEffect();
	}

	/**
	 * Sets the current question. This will be used to check if the player
	 * has answered correctly
	 * @param Question 
	 * @see QuestionPanel
	 */
	public void setCurrentQuestion(Question currentQuestion){
		this.currentQuestion = currentQuestion;
	}
	
	/**
	 * Given the input, check if the answer is correct
	 * @param rightAnswerId- the answer id(this is the location to the answers in 
	 * the questionPanel
	 * @param rightAnswerText - the correct answer, as a string text
	 * @see QuestionPanel
	 */
	public void checkRightAnswer(String answerText, int timeLeft){
		String correctAnswer = "";
		for(Answer ans:currentQuestion.getAnswers()){
			if(ans.isCorrectAnswer()){
				correctAnswer=ans.getAnswerText();
			}
		}
		System.out.println("Player answered:"+answerText+ " ... Correct answer is:"+correctAnswer);
		if(correctAnswer.equals(answerText)){
			// if the answer is correct, increase the number of correct answers
			numberOfCorrectAnswers ++;
			updateTimeUsed(timeLeft);
		}
		else{
			updateTimeUsed(0);
		}
		//after someone has answered, the information about the question is obsolete
		//we therefore resets the question information
		this.currentQuestion = null;
		this.numberOfQuestionsLeftToAnswer = this.numberOfQuestionsLeftToAnswer -1;
		}
	
	/**
	 * Checks if the chosen attack have been completed.
	 * (i.e. the client have answered on enough questions)
	 * @return true if the the client have answered on enough questions,
	 * false otherwise
	 */
	public boolean checkAttackComplete(){
		if(this.numberOfQuestionsLeftToAnswer > 0){
			return false;
		}
		else if(this.numberOfQuestionsLeftToAnswer == 0){
			return true;
		}
		return false;
	}
	
	/**
	 * Checks if the player has answered enough questions to successfully make an attack
	 * @return true if the attack can be carried out, false otherwise
	 */
	public boolean checkIfAttackSuccede(){
		System.out.println("Needed "+this.numberOfCorrectAnswersNeededToScuccessfullyMakeAnAttack+" Answered"+numberOfCorrectAnswers);
		if(this.numberOfCorrectAnswersNeededToScuccessfullyMakeAnAttack <= numberOfCorrectAnswers){
			return true;
		}
		return false;
	}
	
	/**
	 * Will update the time used
	 * @param timeLeft - the time left when the player answered
	 */
	private void updateTimeUsed(int usedTime){
		this.usedTime += usedTime;
		totalAvailableTime += availableTimeLastQuestion;
	}

	public String getTopic() {
		return topic;
	}

	public int getQuestionLevel() {
		return attackSkill.getQuestionLevel();
	}
	
//	/**
//	 * Will be used to do all updates when creating a new question
//	 * @param correctAnswerid - the id to the correct answer
//	 * @param correctAnswerText - the answerText to the correct answer
//	 */
//	public void updateAll(int correctAnswerid, String correctAnswerText){
//		this.correctAnswerId = correctAnswerid;
//		this.correctAnswerText = correctAnswerText;
//		}

	public DamageType getDamageType() {
		return attackSkill.getDamageType();
	}
	
	/**
	 * calulates the damage
	 * @return - the damage without any bonuses
	 */
	public int calculateDamage(){
		double damageModifier = (double)usedTime / (double)totalAvailableTime;
		int damage = attackSkill.getMaxDamage() - attackSkill.getMinDamage();
		return (attackSkill.getMinDamage())+(int)((damage*damageModifier)+0.5);  
	}

	public String getAttackedTarget() {
		return attackedTarget;
	}

	public AttributeTypes getAttriubute() {
		return attackSkill.getAttribute();
	}
}