package worldServer.messageHandler;


import shared.logger.LogSystem;
import shared.network.ClientSocket;
import shared.network.MessageHandler;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.PlayerVerifyMessage;
import shared.network.messages.RequestZoneMessage;
import shared.network.messages.actions.ActionMessage;
import shared.network.messages.actions.MovementMessage;
import shared.network.messages.actions.UpdatePlacementMessage;
import shared.models.Attack;
import shared.models.Item;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.RespawnPoint;
import shared.models.SimpleZone;
import worldServer.serverControls.ChatSystem;
import worldServer.serverControls.ServerControl;
/**
 * ClientHandler holds a connection to a client, and handles the messages received, and sends messages to the client.
 * @author THOR
 *
 */
public class ClientHandler implements MessageHandler{
	private ClientSocket socket;
	private String clientId;
	private ClientState clientState;
	private PlayableCharacter clientCharacter;
	private String inZone;
	private MessageHandlerForWorld gameWorld;
	
	private String charName;
	/**
	 * Constructor, creates a ClientHandler with a ClientSocket and a unique clientId
	 * @param socket
	 * @param clientId
	 */
	public ClientHandler(ClientSocket socket,String clientId){
		clientState = ClientState.CONNECTED;
		this.socket = socket;
		this.socket.addMessageListener(this);
		this.clientId = clientId;
		
		
	}

	/**
	 * Adds a GameWorld as messageListener, so when messages are received from the network, they can be forwarded to the GameWorld.
	 * @param game
	 */
	public void addMessageListener(MessageHandlerForWorld game){
		this.gameWorld = game;
	}
	/**
	 * Return the id of the zone the client is currently in.
	 * @return
	 */
	public String getInZone() {
		return inZone;
	}
	public void setInZone(String inZone) {
		this.inZone = inZone;
	}
	
	/**
	 * Sends a message to the client over the network
	 * @param msg
	 */
	public void sendMessage(Message msg){
		socket.sendMessage(msg);
	}

	/**
	 * Disconnects the client.
	 * @return
	 */
	private boolean disconnectClient(){
		socket.disconnect();
		return true;
	}
	
	/**
	 * Returns the unique clientId
	 * @return
	 */
	public String getClientId() {
		return clientId;
	}

	public void setClientId(String clientId) {
		this.clientId = clientId;
	}
	/**
	 * Is run when the player has been verified, changes its state to Logged in.
	 */
	public void isVerified(){
		writeLog("Player "+clientId+" was verified");
		this.clientState = ClientState.LOGGED_IN;
	}
	/**
	 * Kicks the player from the server, and sends a message with the reason.
	 * @param reason
	 */
	public void kickPlayer(String reason){
		writeLog("Kicked player: "+clientId);
		sendMessage(new Message("WORLDSERVER",clientId,reason,MessageType.CLOSE_CONNECTION));
		disconnectClient();
	}
	
	/**
	 * Receives messages from the client through the network
	 */
	@Override
	public void getMessage(Message msg) {
		msg.printMessage();
		switch(clientState){
		case CONNECTED:
			switch(msg.getMessageType()){
			case VERIFY_PLAYER:
				
					writeLog("Received verifyMessage from:"+clientId);
					PlayerVerifyMessage verifyMsg = (PlayerVerifyMessage)msg;
				if(verifyMsg.getPlayerSessionId()!=null){
					charName = verifyMsg.getCharacterName();
					ServerControl.getServerControl().changeClientSessionId(this.clientId,verifyMsg.getPlayerSessionId());
					ServerControl.getServerControl().sendMsgToLobby(new PlayerVerifyMessage(ServerControl.getServerControl().getSessionId(),"LOBBYSERVER","Verify this player",verifyMsg.getPlayerSessionId(),verifyMsg.getPlayerName(),verifyMsg.getCharacterName()));
				}else{
					writeLog("Player does not have a session id and will be kicked.");
					ServerControl.getServerControl().kickPlayer(clientId, "Player has not been authorized by lobby.");
				}
				break;
			}
			break;
		case LOGGED_IN:
			switch(msg.getMessageType()){
			case CHATMESSAGE:
				gameWorld.getMessage(inZone,msg);
				break;
			case ACTION_MESSAGE:
				gameWorld.getMessage(inZone,msg);
				break;
			case REQUEST_ZONE:
				String zoneId = ((RequestZoneMessage)msg).getZoneId();
				this.setInZone(zoneId);
				SimpleZone zone = gameWorld.getSimpleZone(zoneId);
				ObjectMessage objMsg = new ObjectMessage("WorldServer",clientId,"Here is the requested zone"+zoneId,MessageType.OBJECT_MESSAGE,zone,ObjectType.SIMPLE_ZONE,clientId);
				this.sendMessage(objMsg);
				System.out.println(zone.getRespawn());
				this.clientCharacter.setCurrentLocation(zone.getRespawn());
				this.clientCharacter.setTargetLocation(zone.getRespawn());
				gameWorld.addClientToZone(clientId, zoneId, this.clientCharacter);
				newPosition(zone.getRespawn());
				break;
			case REQUEST_QUESTION:
				gameWorld.getMessage(inZone, msg);
				break;
			case ANSWER_QUESTION:
				gameWorld.getMessage(inZone,msg);
				break;
			}
	
			break;
		case DISCONNECTED:
			break;
		}
	}
	
	public void newPosition(Location currentLocation){
		UpdatePlacementMessage updMsg = new UpdatePlacementMessage("WorldServer",clientId,"New position for player",charName,currentLocation);
		this.sendMessage(updMsg);
	}
	public void movedToZone(String newZoneId){
		this.setInZone(newZoneId);
		SimpleZone zone = gameWorld.getSimpleZone(newZoneId);
		ObjectMessage objMsg = new ObjectMessage("WorldServer",clientId,"Here is the requested zone"+newZoneId,MessageType.OBJECT_MESSAGE,zone,ObjectType.SIMPLE_ZONE,clientId);
		this.sendMessage(objMsg);
	}
	/**
	 * Returns the name of the clients character
	 * @return
	 */
	public String getCharName() {
		return charName;
	}
	/**
	 * Returns the character of the client
	 * @return PlayableCharacter
	 */
	public PlayableCharacter getClientCharacter() {
		return clientCharacter;
	}
	
	/**
	 * For testing, check if received attack lists
	 */
	private void printAttacks(){
	
		if(clientCharacter.getListOfPhysicalAttacks() !=null){
			for(Attack att: clientCharacter.getListOfPhysicalAttacks()){
				writeLog(att.getName());
			}
		}else{
			writeLog("Physical is null");
		}
		if(clientCharacter.getListOfMagicalAttacks() !=null){
			for(Attack att: clientCharacter.getListOfMagicalAttacks()){
				writeLog(att.getName());
			}
		}else{
			writeLog("Magical is null");
		}
	}
	/**
	 * For testing print out inventory
	 * @param clientCharacter
	 */
	private void printInventory(){
		if(clientCharacter.getInventory()!=null){
			for(Item thaThing: clientCharacter.getInventory()){
				writeLog(thaThing.getName());
			}
		}
	}
	
	/**
	 * Sets the character
	 * @param clientCharacter
	 */
	public void setClientCharacter(PlayableCharacter clientCharacter) {
		this.clientCharacter = clientCharacter;
	
		ServerControl.getServerControl().movePlayerToWorld(clientId);
		ObjectMessage msg = new ObjectMessage("WorldServer",clientId,"You are successfully logged in. Here is your character.",MessageType.OBJECT_MESSAGE,clientCharacter,ObjectType.CHARACTER_OBJECT,clientId);
		
		sendMessage(msg);
		printAttacks();
		printInventory();
	}
	/**
	 * Write a message to a log file.
	 * @param logMessage
	 */
	private static void writeLog(String logMessage){
		LogSystem.getInstance().writeWorldLog(logMessage);
	}

	
	/**
	 * When the network connection is lost with the client, this method is run.
	 */
	public void lostConnection() {
		writeLog("Lost connection with: "+clientId+" And removed client from list");
		if(gameWorld!=null){
			gameWorld.lostConnection(clientId);
		}else{
			ServerControl.getServerControl().removePlayer(clientId);
		}
		
	}
}
